@@ -36,26 +36,15 @@ import subprocess
 import urllib
 import errno
 import select
-import sha
 import base64
 import sys
+import hashlib
+
 from encodings.punycode import punycode_encode
 
 from i18n import Q_
 from i18n import ngettext
 
-try:
-	# Python 2.5
-	import hashlib
-	hash_md5  = hashlib.md5
-	hash_sha1 = hashlib.sha1
-except ImportError:
-	# Python 2.4
-	import md5
-	import sha
-	hash_md5 = md5.new
-	hash_sha1 = sha.new
-
 try:
 	import winsound # windows-only built-in module for playing wav
 	import win32api
@@ -133,7 +122,7 @@ def parse_resource(resource):
 	'''Perform stringprep on resource and return it'''
 	if resource:
 		try:
-			from xmpp_stringprep import resourceprep
+			from xmpp.stringprepare import resourceprep
 			return resourceprep.prepare(unicode(resource))
 		except UnicodeError:
 			raise InvalidFormat, 'Invalid character in resource.'
@@ -145,7 +134,7 @@ def prep(user, server, resource):
 
 	if user:
 		try:
-			from xmpp_stringprep import nodeprep
+			from xmpp.stringprepare import nodeprep
 			user = nodeprep.prepare(unicode(user))
 		except UnicodeError:
 			raise InvalidFormat, _('Invalid character in username.')
@@ -156,14 +145,14 @@ def prep(user, server, resource):
 		raise InvalidFormat, _('Server address required.')
 	else:
 		try:
-			from xmpp_stringprep import nameprep
+			from xmpp.stringprepare import nameprep
 			server = nameprep.prepare(unicode(server))
 		except UnicodeError:
 			raise InvalidFormat, _('Invalid character in hostname.')
 
 	if resource:
 		try:
-			from xmpp_stringprep import resourceprep
+			from xmpp.stringprepare import resourceprep
 			resource = resourceprep.prepare(unicode(resource))
 		except UnicodeError:
 			raise InvalidFormat, _('Invalid character in resource.')
@@ -181,6 +170,11 @@ def prep(user, server, resource):
 		else:
 			return server
 
+def windowsify(s):
+	if os.name == 'nt':
+		return s.capitalize()
+	return s
+
 def temp_failure_retry(func, *args, **kwargs):
 	while True:
 		try:
@@ -285,10 +279,10 @@ def get_uf_role(role, plural = False):
 		else:
 			role_name = _('Visitor')
 	return role_name
-	
+
 def get_uf_affiliation(affiliation):
 	'''Get a nice and translated affilition for muc'''
-	if affiliation == 'none': 
+	if affiliation == 'none':
 		affiliation_name = Q_('?Group Chat Contact Affiliation:None')
 	elif affiliation == 'owner':
 		affiliation_name = _('Owner')
@@ -344,33 +338,25 @@ def get_uf_chatstate(chatstate):
 		return _('has closed the chat window or tab')
 	return ''
 
-def is_in_path(name_of_command, return_abs_path = False):
-	# if return_abs_path is True absolute path will be returned
-	# for name_of_command
-	# on failures False is returned
-	is_in_dir = False
-	found_in_which_dir = None
-	path = os.getenv('PATH').split(os.pathsep)
-	for path_to_directory in path:
-		try:
-			contents = os.listdir(path_to_directory)
-		except OSError: # user can have something in PATH that is not a dir
-			pass
-		else:
-			is_in_dir = name_of_command in contents
-		if is_in_dir:
-			if return_abs_path:
-				found_in_which_dir = path_to_directory
-			break
+def is_in_path(command, return_abs_path=False):
+	'''Returns True if 'command' is found in one of the directories in the
+		user's path. If 'return_abs_path' is True, returns the absolute path of
+		the first found command instead. Returns False otherwise and on errors.'''
 
-	if found_in_which_dir:
-		abs_path = os.path.join(path_to_directory, name_of_command)
-		return abs_path
-	else:
-		return is_in_dir
+	for directory in os.getenv('PATH').split(os.pathsep):
+		try:
+			if command in os.listdir(directory):
+				if return_abs_path:
+					return os.path.join(directory, command)
+				else:
+					return True
+		except OSError:
+			# If the user has non directories in his path
+			pass
+	return False
 
 def exec_command(command):
-	subprocess.Popen(command, shell = True)
+	subprocess.Popen('%s &' % command, shell=True).wait()
 
 def build_command(executable, parameter):
 	# we add to the parameter (can hold path with spaces)
@@ -386,11 +372,7 @@ def get_file_path_from_dnd_dropped_uri(uri):
 	if re.match('^file:///[a-zA-Z]:/', path): # windows
 		path = path[8:] # 8 is len('file:///')
 	elif path.startswith('file://'): # nautilus, rox
-		if sys.platform == 'darwin':
-			# OS/X includes hostname in file:// URI
-			path = re.sub('file://[^/]*', '', path)
-		else:
-			path = path[7:] # 7 is len('file://')
+		path = path[7:] # 7 is len('file://')
 	elif path.startswith('file:'): # xffm
 		path = path[5:] # 5 is len('file:')
 	return path
@@ -503,11 +485,11 @@ def get_documents_path():
 	return path
 
 def sanitize_filename(filename):
-	'''makes sure the filename we will write does contain only acceptable and 
+	'''makes sure the filename we will write does contain only acceptable and
 	latin characters, and is not too long (in that case hash it)'''
 	# 48 is the limit
 	if len(filename) > 48:
-		hash = hash_md5(filename)
+		hash = hashlib.md5(filename)
 		filename = base64.b64encode(hash.digest())
 
 	filename = punycode_encode(filename) # make it latin chars only
@@ -516,7 +498,7 @@ def sanitize_filename(filename):
 		filename = filename.replace('?', '_').replace(':', '_')\
 			.replace('\\', '_').replace('"', "'").replace('|', '_')\
 			.replace('*', '_').replace('<', '_').replace('>', '_')
-	
+
 	return filename
 
 def reduce_chars_newlines(text, max_chars = 0, max_lines = 0):
@@ -539,7 +521,7 @@ def reduce_chars_newlines(text, max_chars = 0, max_lines = 0):
 		lines = text.split('\n', max_lines)[:max_lines]
 	if max_chars > 0:
 		if lines:
-			lines = map(lambda e: _cut_if_long(e), lines)
+			lines = [_cut_if_long(e) for e in lines]
 	if lines:
 		reduced_text = '\n'.join(lines)
 		if reduced_text != text:
@@ -614,7 +596,7 @@ def sort_dataforms_func(d1, d2):
 
 def compute_caps_hash(identities, features, dataforms=[], hash_method='sha-1'):
 	'''Compute caps hash according to XEP-0115, V1.5
-	
+
 	dataforms are xmpp.DataForms objects as common.dataforms don't allow several
 	values without a field type list-multi'''
 	S = ''
@@ -638,29 +620,25 @@ def compute_caps_hash(identities, features, dataforms=[], hash_method='sha-1'):
 		if form_type:
 			S += form_type.getValue() + '<'
 			del fields['FORM_TYPE']
-		vars = sorted(fields.keys())
-		for var in vars:
+		for var in sorted(fields.keys()):
 			S += '%s<' % var
 			values = sorted(fields[var].getValues())
 			for value in values:
 				S += '%s<' % value
 
 	if hash_method == 'sha-1':
-		hash = hash_sha1(S)
+		hash_ = hashlib.sha1(S)
 	elif hash_method == 'md5':
-		hash = hash_md5(S)
+		hash_ = hashlib.md5(S)
 	else:
 		return ''
-	return base64.b64encode(hash.digest())
+	return base64.b64encode(hash_.digest())
 
-# import gajim only when needed (after decode_string is defined) see #4764 
+# import gajim only when needed (after decode_string is defined) see #4764
 
 import gajim
+import pep
 
-try:
-	from osx import nsapp
-except ImportError:
-	pass
 
 def convert_bytes(string):
 	suffix = ''
@@ -729,7 +707,7 @@ def launch_browser_mailer(kind, uri):
 			pass
 
 	else:
-		if kind == 'mail' and not uri.startswith('mailto:'):
+		if kind in ('mail', 'sth_at_sth') and not uri.startswith('mailto:'):
 			uri = 'mailto:' + uri
 
 		if kind == 'url' and uri.startswith('www.'):
@@ -741,13 +719,10 @@ def launch_browser_mailer(kind, uri):
 			command = 'kfmclient exec'
 		elif gajim.config.get('openwith') == 'exo-open':
 			command = 'exo-open'
-		elif ((sys.platform == 'darwin') and\
-		(gajim.config.get('openwith') == 'open')):
-			command = 'open'
 		elif gajim.config.get('openwith') == 'custom':
 			if kind == 'url':
 				command = gajim.config.get('custombrowser')
-			if kind == 'mail':
+			elif kind in ('mail', 'sth_at_sth'):
 				command = gajim.config.get('custommailapp')
 			if command == '': # if no app is configured
 				return
@@ -771,9 +746,6 @@ def launch_file_manager(path_to_open):
 			command = 'kfmclient exec'
 		elif gajim.config.get('openwith') == 'exo-open':
 			command = 'exo-open'
-		elif ((sys.platform == 'darwin') and\
-		(gajim.config.get('openwith') == 'open')):
-			command = 'open'
 		elif gajim.config.get('openwith') == 'custom':
 			command = gajim.config.get('custom_file_manager')
 		if command == '': # if no app is configured
@@ -790,17 +762,53 @@ def play_sound(event):
 	path_to_soundfile = gajim.config.get_per('soundevents', event, 'path')
 	play_sound_file(path_to_soundfile)
 
+def check_soundfile_path(file,
+								 dirs=(gajim.gajimpaths.root, gajim.DATA_DIR)):
+	'''Check if the sound file exists.
+	:param file: the file to check, absolute or relative to 'dirs' path
+	:param dirs: list of knows paths to fallback if the file doesn't exists
+					 (eg: ~/.gajim/sounds/, DATADIR/sounds...).
+	:return      the path to file or None if it doesn't exists.'''
+	if not file:
+		return None
+	elif os.path.exists(file):
+		return file
+
+	for d in dirs:
+		d = os.path.join(d, 'sounds', file)
+		if os.path.exists(d):
+			return d
+	return None
+
+def strip_soundfile_path(file,
+								 dirs=(gajim.gajimpaths.root, gajim.DATA_DIR),
+								 abs=True):
+	'''Remove knowns paths from a sound file:
+	Filechooser returns absolute path. If path is a known fallback path, we remove it.
+	So config have no hardcoded path	to DATA_DIR and text in textfield is shorther.
+	param: file: the filename to strip.
+	param: dirs: list of knowns paths from which the filename should be stripped.
+	param:  abs: force absolute path on dirs
+	'''
+	if not file:
+		return None
+
+	name = os.path.basename(file)
+	for d in dirs:
+		d = os.path.join(d, 'sounds', name)
+		if abs:
+			d = os.path.abspath(d)
+		if file == d:
+			return name
+	return file
+
 def play_sound_file(path_to_soundfile):
 	if path_to_soundfile == 'beep':
 		exec_command('beep')
 		return
-	if path_to_soundfile is None or not os.path.exists(path_to_soundfile):
+	path_to_soundfile = check_soundfile_path(path_to_soundfile)
+	if path_to_soundfile is None:
 		return
-	if sys.platform == 'darwin':
-		try:
-			nsapp.playFile(path_to_soundfile)
-		except NameError:
-			pass
 	elif os.name == 'nt':
 		try:
 			winsound.PlaySound(path_to_soundfile,
@@ -818,7 +826,7 @@ def get_global_show():
 	maxi = 0
 	for account in gajim.connections:
 		if not gajim.config.get_per('accounts', account,
-			'sync_with_global_status'):
+		'sync_with_global_status'):
 			continue
 		connected = gajim.connections[account].connected
 		if connected > maxi:
@@ -829,7 +837,7 @@ def get_global_status():
 	maxi = 0
 	for account in gajim.connections:
 		if not gajim.config.get_per('accounts', account,
-			'sync_with_global_status'):
+		'sync_with_global_status'):
 			continue
 		connected = gajim.connections[account].connected
 		if connected > maxi:
@@ -837,7 +845,55 @@ def get_global_status():
 			status = gajim.connections[account].status
 	return status
 
-def statuses_unified(): 
+def get_pep_dict(account):
+	pep_dict = {}
+	con = gajim.connections[account]
+	# activity
+	if 'activity' in con.activity and con.activity['activity'] in pep.ACTIVITIES:
+		activity = con.activity['activity']
+		if 'subactivity' in con.activity and con.activity['subactivity'] in \
+		pep.ACTIVITIES[activity]:
+			subactivity = con.activity['subactivity']
+		else:
+			subactivity = 'other'
+	else:
+		activity = ''
+		subactivity = ''
+	if 'text' in con.activity:
+		text = con.activity['text']
+	else:
+		text = ''
+	pep_dict['activity'] = activity
+	pep_dict['subactivity'] = subactivity
+	pep_dict['activity_text'] = text
+
+	# mood
+	if 'mood' in con.mood and con.mood['mood'] in pep.MOODS:
+		mood = con.mood['mood']
+	else:
+		mood = ''
+	if 'text' in con.mood:
+		text = con.mood['text']
+	else:
+		text = ''
+	pep_dict['mood'] = mood
+	pep_dict['mood_text'] = text
+	return pep_dict
+
+def get_global_pep():
+	maxi = 0
+	pep_dict = {'activity': '', 'mood': ''}
+	for account in gajim.connections:
+		if not gajim.config.get_per('accounts', account,
+		'sync_with_global_status'):
+			continue
+		connected = gajim.connections[account].connected
+		if connected > maxi:
+			maxi = connected
+			pep_dict = get_pep_dict(account)
+	return pep_dict
+
+def statuses_unified():
 	'''testing if all statuses are the same.'''
 	reference = None
 	for account in gajim.connections:
@@ -890,7 +946,7 @@ def get_jid_from_iq(iq_obj):
 
 def get_auth_sha(sid, initiator, target):
 	''' return sha of sid + initiator + target used for proxy auth'''
-	return sha.new("%s%s%s" % (sid, initiator, target)).hexdigest()
+	return hashlib.sha1("%s%s%s" % (sid, initiator, target)).hexdigest()
 
 def remove_invalid_xml_chars(string):
 	if string:
@@ -925,10 +981,10 @@ def get_random_string_16():
 	''' create random string of length 16'''
 	rng = range(65, 90)
 	rng.extend(range(48, 57))
-	char_sequence = map(lambda e:chr(e), rng)
+	char_sequence = [chr(e) for e in rng]
 	from random import sample
 	return ''.join(sample(char_sequence, 16))
-	
+
 def get_os_info():
 	if gajim.os_info:
 		return gajim.os_info
@@ -957,9 +1013,9 @@ def get_os_info():
 		full_path_to_executable = is_in_path(executable, return_abs_path = True)
 		if full_path_to_executable:
 			command = executable + params
-			p = subprocess.Popen([command], shell=True, stdin=subprocess.PIPE, 
-				stdout=subprocess.PIPE, close_fds=True) 
-			p.wait() 
+			p = subprocess.Popen([command], shell=True, stdin=subprocess.PIPE,
+				stdout=subprocess.PIPE, close_fds=True)
+			p.wait()
 			output = temp_failure_retry(p.stdout.readline).strip()
 			# some distros put n/a in places, so remove those
 			output = output.replace('n/a', '').replace('N/A', '')
@@ -1005,6 +1061,7 @@ def get_os_info():
 	gajim.os_info = os_info
 	return os_info
 
+
 def allow_showing_notification(account, type_ = 'notify_on_new_message',
 advanced_notif_num = None, is_first_message = True):
 	'''is it allowed to show nofication?
@@ -1042,7 +1099,7 @@ def allow_popup_window(account, advanced_notif_num = None):
 		return True
 	return False
 
-def allow_sound_notification(sound_event, advanced_notif_num = None):
+def allow_sound_notification(account, sound_event, advanced_notif_num=None):
 	if advanced_notif_num is not None:
 		sound = gajim.config.get_per('notifications', str(advanced_notif_num),
 			'sound')
@@ -1050,7 +1107,9 @@ def allow_sound_notification(sound_event, advanced_notif_num = None):
 			return True
 		if sound == 'no':
 			return False
-	if gajim.config.get_per('soundevents', sound_event, 'enabled'):
+	if gajim.config.get('sounddnd') or gajim.connections[account].connected != \
+	gajim.SHOW_LIST.index('dnd') and gajim.config.get_per('soundevents',
+	sound_event, 'enabled'):
 		return True
 	return False
 
@@ -1076,7 +1135,7 @@ def get_chat_control(account, contact):
 		return gajim.interface.msg_win_mgr.get_control(contact.jid, account)
 
 def get_notification_icon_tooltip_dict():
-	'''returns a dict of the form {acct: {'show': show, 'message': message, 
+	'''returns a dict of the form {acct: {'show': show, 'message': message,
 	'event_lines': [list of text lines to show in tooltip]}'''
 	# How many events must there be before they're shown summarized, not per-user
 	max_ungrouped_events = 10
@@ -1124,7 +1183,7 @@ def get_notification_icon_tooltip_dict():
 					else:
 						text += _(' from %s') % (jid)
 					account['event_lines'].append(text)
-		
+
 		# Display unseen events numbers, if any
 		if total_non_messages > 0:
 			if total_non_messages > max_ungrouped_events:
@@ -1132,7 +1191,7 @@ def get_notification_icon_tooltip_dict():
 					'%d event pending',
 					'%d events pending',
 					total_non_messages, total_non_messages, total_non_messages)
-				accounts[account]['event_lines'].append(text)
+				account['event_lines'].append(text)
 			else:
 				for jid in non_messages.keys():
 					text = ngettext(
@@ -1140,14 +1199,14 @@ def get_notification_icon_tooltip_dict():
 						'%d events pending',
 						non_messages[jid], non_messages[jid], non_messages[jid])
 					text += _(' from user %s') % (jid)
-					accounts[account]['event_lines'].append(text)
+					account[account]['event_lines'].append(text)
 
 	return accounts
 
 def get_notification_icon_tooltip_text():
 	text = None
 	# How many events must there be before they're shown summarized, not per-user
-	max_ungrouped_events = 10
+	# max_ungrouped_events = 10
 	# Character which should be used to indent in the tooltip.
 	indent_with = ' '
 
@@ -1168,7 +1227,7 @@ def get_notification_icon_tooltip_text():
 	# If there is only one account, its status is shown on the first line.
 	if show_more_accounts:
 		text = _('Gajim')
-	else:		
+	else:
 		text = _('Gajim - %s') % (get_account_status(accounts[0]))
 
 	# Gather and display events. (With accounts, when there are more.)
@@ -1205,10 +1264,11 @@ def get_accounts_info():
 			message = message.strip()
 		if message != '':
 			single_line += ': ' + message
-		accounts.append({'name': account, 'status_line': single_line, 
+		accounts.append({'name': account, 'status_line': single_line,
 				'show': status, 'message': message})
 	return accounts
 
+
 def get_iconset_path(iconset):
 	if os.path.isdir(os.path.join(gajim.DATA_DIR, 'iconsets', iconset)):
 		return os.path.join(gajim.DATA_DIR, 'iconsets', iconset)
@@ -1242,13 +1302,13 @@ def prepare_and_validate_gpg_keyID(account, jid, keyID):
 	'''Returns an eight char long keyID that can be used with for GPG encryption with this contact.
 	If the given keyID is None, return UNKNOWN; if the key does not match the assigned key
 	XXXXXXXXMISMATCH is returned. If the key is trusted and not yet assigned, assign it'''
-	if gajim.connections[account].USE_GPG:	
+	if gajim.connections[account].USE_GPG:
 		if keyID and len(keyID) == 16:
 			keyID = keyID[8:]
-		
+
 		attached_keys = gajim.config.get_per('accounts', account,
 			'attached_gpg_keys').split()
-		
+
 		if jid in attached_keys and keyID:
 			attachedkeyID = attached_keys[attached_keys.index(jid) + 1]
 			if attachedkeyID != keyID:
@@ -1257,7 +1317,7 @@ def prepare_and_validate_gpg_keyID(account, jid, keyID):
 		elif jid in attached_keys:
 			# An unsigned presence, just use the assigned key
 			keyID = attached_keys[attached_keys.index(jid) + 1]
-		elif keyID: 
+		elif keyID:
 			public_keys = gajim.connections[account].ask_gpg_keys()
 			# Assign the corresponding key, if we have it in our keyring
 			if keyID in public_keys:
