@@ -5,6 +5,7 @@
 ## Copyright (C) 2004 Vincent Hanquez <tab AT snarc.org>
 ## Copyright (C) 2004-2007 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2009 Benjamin Richter <br AT waldteufel-online.net>
 ##
 ## This file is part of Gajim.
 ##
@@ -25,6 +26,23 @@ import locale
 import gettext
 import os
 import defs
+import unicodedata
+
+def paragraph_direction_mark(text):
+	"""
+	Determine paragraph writing direction according to
+	http://www.unicode.org/reports/tr9/#The_Paragraph_Level
+	
+	Returns either Unicode LTR mark or RTL mark.
+	"""
+	for char in text:
+		bidi = unicodedata.bidirectional(char)
+		if bidi == 'L':
+			return u'\u200E'
+		elif bidi == 'AL' or bidi == 'R':
+			return u'\u200F'
+
+	return u'\u200E'
 
 APP = 'gajim'
 DIR = defs.localedir
@@ -56,7 +74,7 @@ def Q_(s):
 	# so we must use as:
 	# s = Q_('?vcard:Unknown')
 	# widget.set_text(s)
-	# Q_() removes the ?vcard: 
+	# Q_() removes the ?vcard:
 	# but gettext while parsing the file detects ?vcard:Unknown as a whole string.
 	# translator can either put the ?vcard: part or no (easier for him or her to no)
 	# nothing fails
@@ -68,7 +86,7 @@ def Q_(s):
 def ngettext(s_sing, s_plural, n, replace_sing = None, replace_plural = None):
 	'''use as:
 	i18n.ngettext('leave room %s', 'leave rooms %s', len(rooms), 'a', 'a, b, c')
-	
+
 	in other words this is a hack to ngettext() to support %s %d etc..
 	'''
 	text = _translation.ungettext(s_sing, s_plural, n)
