@@ -24,6 +24,8 @@
 import xmpp
 import gajim
 import connection_handlers
+import logging
+log = logging.getLogger('gajim.c.pubsub')
 
 class ConnectionPubSub:
 	def __init__(self):
@@ -33,12 +35,12 @@ class ConnectionPubSub:
 		if not self.connection or self.connected < 2:
 			return
 		query = xmpp.Iq('get', to=jid)
-		pb = query.addChild('pubsub', {'xmlns': xmpp.NS_PUBSUB})
+		pb = query.addChild('pubsub', namespace=xmpp.NS_PUBSUB)
 		pb.addChild('subscriptions')
 
-		id = self.connection.send(query)
+		id_ = self.connection.send(query)
 
-		self.__callbacks[id]=(cb, args, kwargs)
+		self.__callbacks[id_]=(cb, args, kwargs)
 
 	def send_pb_subscribe(self, jid, node, cb, *args, **kwargs):
 		if not self.connection or self.connected < 2:
@@ -48,9 +50,9 @@ class ConnectionPubSub:
 		pb = query.addChild('pubsub', namespace=xmpp.NS_PUBSUB)
 		pb.addChild('subscribe', {'node': node, 'jid': our_jid})
 
-		id = self.connection.send(query)
+		id_ = self.connection.send(query)
 
-		self.__callbacks[id]=(cb, args, kwargs)
+		self.__callbacks[id_]=(cb, args, kwargs)
 
 	def send_pb_unsubscribe(self, jid, node, cb, *args, **kwargs):
 		if not self.connection or self.connected < 2:
@@ -60,21 +62,36 @@ class ConnectionPubSub:
 		pb = query.addChild('pubsub', namespace=xmpp.NS_PUBSUB)
 		pb.addChild('unsubscribe', {'node': node, 'jid': our_jid})
 
-		id = self.connection.send(query)
+		id_ = self.connection.send(query)
 
-		self.__callbacks[id]=(cb, args, kwargs)
+		self.__callbacks[id_]=(cb, args, kwargs)
 
-	def send_pb_publish(self, jid, node, item, id_):
+	def send_pb_publish(self, jid, node, item, id_, options=None):
 		'''Publish item to a node.'''
 		if not self.connection or self.connected < 2:
 			return
 		query = xmpp.Iq('set', to=jid)
 		e = query.addChild('pubsub', namespace=xmpp.NS_PUBSUB)
-		e = e.addChild('publish', {'node': node})
-		e = e.addChild('item', {'id': id_}, [item])	# TODO: we should generate id... or we shouldn't?
+		p = e.addChild('publish', {'node': node})
+		p.addChild('item', {'id': id_}, [item])
+		if options:
+			p = e.addChild('publish-options')
+			p.addChild(node=options)
 
 		self.connection.send(query)
 
+	def send_pb_retrieve(self, jid, node, cb=None, *args, **kwargs): 
+		'''Get items from a node''' 
+		if not self.connection or self.connected < 2: 
+			return 
+		query = xmpp.Iq('get', to=jid) 
+		r = query.addChild('pubsub', namespace=xmpp.NS_PUBSUB) 
+		r = r.addChild('items', {'node': node}) 
+		id_ = self.connection.send(query)
+
+		if cb:
+			self.__callbacks[id_]=(cb, args, kwargs)
+
 	def send_pb_retract(self, jid, node, id_):
 		'''Delete item from a node'''
 		if not self.connection or self.connected < 2:
@@ -129,21 +146,49 @@ class ConnectionPubSub:
 		self.connection.send(query)
 
 	def _PubSubCB(self, conn, stanza):
+		log.debug('_PubsubCB')
 		try:
 			cb, args, kwargs = self.__callbacks.pop(stanza.getID())
 			cb(conn, stanza, *args, **kwargs)
 		except Exception:
 			pass
 
+		pubsub = stanza.getTag('pubsub')
+		if not pubsub:
+			return
+		items = pubsub.getTag('items')
+		if not items:
+			return
+		item = items.getTag('item')
+		if not item:
+			return
+		storage = item.getTag('storage')
+		if storage:
+			ns = storage.getNamespace()
+			if ns == 'storage:bookmarks':
+				self._parse_bookmarks(storage, 'pubsub')
+
+	def _PubSubErrorCB(self, conn, stanza):
+		log.debug('_PubsubErrorCB')
+		pubsub = stanza.getTag('pubsub')
+		if not pubsub:
+			return
+		items = pubsub.getTag('items')
+		if not items:
+			return
+		if items.getAttr('node') == 'storage:bookmarks':
+			# Receiving bookmarks from pubsub failed, so take them from xml
+			self.get_bookmarks(storage_type='xml')
+
 	def request_pb_configuration(self, jid, node):
 		if not self.connection or self.connected < 2:
 			return
 		query = xmpp.Iq('get', to=jid)
 		e = query.addChild('pubsub', namespace=xmpp.NS_PUBSUB_OWNER)
 		e = e.addChild('configure', {'node': node})
-		id = self.connection.getAnID()
-		query.setID(id)
-		self.awaiting_answers[id] = (connection_handlers.PEP_CONFIG,)
+		id_ = self.connection.getAnID()
+		query.setID(id_)
+		self.awaiting_answers[id_] = (connection_handlers.PEP_CONFIG,)
 		self.connection.send(query)
 
 # vim: se ts=3:
