@@ -2,7 +2,6 @@
 ## src/common/stanza_session.py
 ##
 ## Copyright (C) 2007 Julien Pivotto <roidelapluie AT gmail.com>
-##                    Stephan Erb <steve-e AT h3c.de>
 ## Copyright (C) 2007-2008 Yann Leboulanger <asterix AT lagaule.org>
 ##                         Brendan Taylor <whateley AT gmail.com>
 ##                         Jean-Marie Traissard <jim AT lapin.org>
@@ -24,29 +23,39 @@
 ##
 
 from common import gajim
-
 from common import xmpp
-from common import exceptions
-
-import random
-import string
-
-import time
-
+from common.exceptions import DecryptionError, NegotiationError
 import xmpp.c14n
 
+import itertools
+import random
+import string
+import time
 import base64
 import os
+from hashlib import sha256
+from hmac import HMAC
+from common import crypto
+
+if gajim.HAVE_PYCRYPTO:
+	from Crypto.Cipher import AES
+	from Crypto.PublicKey import RSA
+
+	from common import dh
+	import secrets
 
 XmlDsig = 'http://www.w3.org/2000/09/xmldsig#'
 
 class StanzaSession(object):
+	'''
+	'''
 	def __init__(self, conn, jid, thread_id, type_):
+		'''
+		'''
 		self.conn = conn
-
 		self.jid = jid
-
 		self.type = type_
+		self.resource = None
 
 		if thread_id:
 			self.received_thread_id = True
@@ -68,9 +77,17 @@ class StanzaSession(object):
 	def is_loggable(self):
 		return self.loggable and gajim.config.should_log(self.conn.name, self.jid)
 
-	# remove events associated with this session from the queue
-	# returns True if any events were removed (unlike gajim.events.remove_events)
+	def get_to(self):
+		to = str(self.jid)
+		if self.resource and not to.endswith(self.resource):
+			to += '/' + self.resource
+		return to
+
 	def remove_events(self, types):
+		'''
+		Remove events associated with this session from the queue.
+		returns True if any events were removed (unlike events.py remove_events)
+		'''
 		any_removed = False
 
 		for j in (self.jid, self.jid.getStripped()):
@@ -91,14 +108,14 @@ class StanzaSession(object):
 		return any_removed
 
 	def generate_thread_id(self):
-		return ''.join([random.choice(string.ascii_letters) for x in xrange(0,
-			32)])
+		return ''.join([f(string.ascii_letters) for f in itertools.repeat(
+			random.choice, 32)])
 
 	def send(self, msg):
 		if self.thread_id:
 			msg.NT.thread = self.thread_id
 
-		msg.setAttr('to', self.jid)
+		msg.setAttr('to', self.get_to())
 		self.conn.send_stanza(msg)
 
 		if isinstance(msg, xmpp.Message):
@@ -123,9 +140,10 @@ class StanzaSession(object):
 		self.cancelled_negotiation()
 
 	def cancelled_negotiation(self):
-		'''A negotiation has been cancelled, so reset this session to its default
-		state.'''
-
+		'''
+		A negotiation has been cancelled, so reset this session to its default
+		state.
+		'''
 		if self.control:
 			self.control.on_cancel_session_negotiation()
 
@@ -155,61 +173,64 @@ class StanzaSession(object):
 		# we could send an acknowledgement message to the remote client here
 		self.status = None
 
-if gajim.HAVE_PYCRYPTO:
-	from Crypto.Cipher import AES
-	from Crypto.Hash import HMAC, SHA256
-	from Crypto.PublicKey import RSA
-	from common import crypto
-
-	from common import dh
-	import secrets
-
-# an encrypted stanza negotiation has several states. i've represented them
-# as the following values in the 'status'
-# attribute of the session object:
-
-# 1. None:
-#				default state
-# 2. 'requested-e2e':
-#				this client has initiated an esession negotiation and is waiting
-#				for a response
-# 3. 'responded-e2e':
-#				this client has responded to an esession negotiation request and
-#				is waiting for the initiator to identify itself and complete the
-#				negotiation
-# 4. 'identified-alice':
-#				this client identified itself and is waiting for the responder to
-#				identify itself and complete the negotiation
-# 5. 'active':
-#				an encrypted session has been successfully negotiated. messages
-#				of any of the types listed in 'encryptable_stanzas' should be
-#				encrypted before they're sent.
-
-# the transition between these states is handled in gajim.py's
-#	handle_session_negotiation method.
 
 class EncryptedStanzaSession(StanzaSession):
+	'''
+	An encrypted stanza negotiation has several states. They arerepresented	as
+	the following values in the 'status' attribute of the session object:
+
+		1. None:
+				default state
+		2. 'requested-e2e':
+				this client has initiated an esession negotiation and is waiting
+				for a response
+		3. 'responded-e2e':
+				this client has responded to an esession negotiation request and
+				is waiting for the initiator to identify itself and complete the
+				negotiation
+ 		4. 'identified-alice':
+				this client identified itself and is waiting for the responder to
+				identify itself and complete the negotiation
+		5. 'active':
+				an encrypted session has been successfully negotiated. messages
+				of any of the types listed in 'encryptable_stanzas' should be
+				encrypted before they're sent.
+
+	The transition between these states is handled in gajim.py's
+	handle_session_negotiation method.
+	'''
 	def __init__(self, conn, jid, thread_id, type_='chat'):
 		StanzaSession.__init__(self, conn, jid, thread_id, type_='chat')
 
 		self.xes = {}
 		self.es = {}
-
 		self.n = 128
-
 		self.enable_encryption = False
 
 		# _s denotes 'self' (ie. this client)
 		self._kc_s = None
-
 		# _o denotes 'other' (ie. the client at the other end of the session)
 		self._kc_o = None
 
 		# has the remote contact's identity ever been verified?
 		self.verified_identity = False
 
-	# keep the encrypter updated with my latest cipher key
+	def _get_contact(self):
+		c = gajim.contacts.get_contact(self.conn.name, self.jid, self.resource)
+		if not c:
+			c = gajim.contacts.get_contact(self.conn.name, self.jid)
+		return c
+
+	def _is_buggy_gajim(self):
+		c = self._get_contact()
+		if gajim.capscache.is_supported(c, xmpp.NS_ROSTERX):
+			return False
+		return True
+
 	def set_kc_s(self, value):
+		'''
+		keep the encrypter updated with my latest cipher key
+		'''
 		self._kc_s = value
 		self.encrypter = self.cipher.new(self._kc_s, self.cipher.MODE_CTR,
 			counter=self.encryptcounter)
@@ -217,8 +238,10 @@ class EncryptedStanzaSession(StanzaSession):
 	def get_kc_s(self):
 		return self._kc_s
 
-	# keep the decrypter updated with the other party's latest cipher key
 	def set_kc_o(self, value):
+		'''
+		keep the decrypter updated with the other party's latest cipher key
+		'''
 		self._kc_o = value
 		self.decrypter = self.cipher.new(self._kc_o, self.cipher.MODE_CTR,
 			counter=self.decryptcounter)
@@ -239,14 +262,14 @@ class EncryptedStanzaSession(StanzaSession):
 
 	def sign(self, string):
 		if self.negotiated['sign_algs'] == (XmlDsig + 'rsa-sha256'):
-			hash = crypto.sha256(string)
-			return crypto.encode_mpi(gajim.pubkey.sign(hash, '')[0])
+			hash_ = crypto.sha256(string)
+			return crypto.encode_mpi(gajim.pubkey.sign(hash_, '')[0])
 
 	def encrypt_stanza(self, stanza):
-		encryptable = filter(lambda x: x.getName() not in ('error', 'amp',
-			'thread'), stanza.getChildren())
+		encryptable = [x for x in stanza.getChildren() if x.getName() not in
+			('error', 'amp', 'thread')]
 
-		# XXX can also encrypt contents of <error/> elements in stanzas @type =
+		# FIXME can also encrypt contents of <error/> elements in stanzas @type =
 		# 'error'
 		# (except for <defined-condition
 		# xmlns='urn:ietf:params:xml:ns:xmpp-stanzas'/> child elements)
@@ -265,7 +288,7 @@ class EncryptedStanzaSession(StanzaSession):
 		c.setNamespace('http://www.xmpp.org/extensions/xep-0200.html#ns')
 		c.NT.data = base64.b64encode(m_final)
 
-		# XXX check for rekey request, handle <key/> elements
+		# FIXME check for rekey request, handle <key/> elements
 
 		m_content = ''.join(map(str, c.getChildren()))
 		c.NT.mac = base64.b64encode(self.hmac(self.km_s, m_content + \
@@ -283,10 +306,10 @@ class EncryptedStanzaSession(StanzaSession):
 		return stanza
 
 	def is_xep_200_encrypted(self, msg):
-		msg.getTag('c', namespace=common.xmpp.NS_STANZA_CRYPTO)
+		msg.getTag('c', namespace=xmpp.NS_STANZA_CRYPTO)
 
 	def hmac(self, key, content):
-		return HMAC.new(key, content, self.hash_alg).digest()
+		return HMAC(key, content, self.hash_alg).digest()
 
 	def generate_initiator_keys(self, k):
 		return (self.hmac(k, 'Initiator Cipher Key'),
@@ -312,7 +335,7 @@ class EncryptedStanzaSession(StanzaSession):
 		return self.encrypter.encrypt(padded)
 
 	def decrypt_stanza(self, stanza):
-		# delete the unencrypted explanation body, if it exists
+		''' delete the unencrypted explanation body, if it exists '''
 		orig_body = stanza.getTag('body')
 		if orig_body:
 			stanza.delChild(orig_body)
@@ -323,15 +346,14 @@ class EncryptedStanzaSession(StanzaSession):
 		stanza.delChild(c)
 
 		# contents of <c>, minus <mac>, minus whitespace
-		macable = ''.join(map(str, filter(lambda x: x.getName() != 'mac',
-			c.getChildren())))
+		macable = ''.join(str(x) for x in c.getChildren() if x.getName() != 'mac')
 
 		received_mac = base64.b64decode(c.getTagData('mac'))
 		calculated_mac = self.hmac(self.km_o, macable + \
 			crypto.encode_mpi_with_padding(self.c_o))
 
 		if not calculated_mac == received_mac:
-			raise exceptions.DecryptionError, 'bad signature'
+			raise DecryptionError('bad signature')
 
 		m_final = base64.b64decode(c.getTagData('data'))
 		m_compressed = self.decrypt(m_final)
@@ -340,7 +362,7 @@ class EncryptedStanzaSession(StanzaSession):
 		try:
 			parsed = xmpp.Node(node='<node>' + plaintext + '</node>')
 		except Exception:
-			raise exceptions.DecryptionError, 'decrypted <data/> not parseable as XML'
+			raise DecryptionError('decrypted <data/> not parseable as XML')
 
 		for child in parsed.getChildren():
 			stanza.addChild(node=child)
@@ -359,14 +381,15 @@ class EncryptedStanzaSession(StanzaSession):
 
 	def get_shared_secret(self, e, y, p):
 		if (not 1 < e < (p - 1)):
-			raise exceptions.NegotiationError, 'invalid DH value'
+			raise NegotiationError('invalid DH value')
 
 		return crypto.sha256(crypto.encode_mpi(crypto.powmod(e, y, p)))
 
 	def c7lize_mac_id(self, form):
 		kids = form.getChildren()
-		macable = filter(lambda x: x.getVar() not in ('mac', 'identity'), kids)
-		return ''.join(map(lambda el: xmpp.c14n.c14n(el), macable))
+		macable = [x for x in kids if x.getVar() not in ('mac', 'identity')]
+		return ''.join(xmpp.c14n.c14n(el, self._is_buggy_gajim()) for el in \
+			macable)
 
 	def verify_identity(self, form, dh_i, sigmai, i_o):
 		m_o = base64.b64decode(form['mac'])
@@ -375,7 +398,8 @@ class EncryptedStanzaSession(StanzaSession):
 		m_o_calculated = self.hmac(self.km_o, crypto.encode_mpi(self.c_o) + id_o)
 
 		if m_o_calculated != m_o:
-			raise exceptions.NegotiationError, 'calculated m_%s differs from received m_%s' % (i_o, i_o)
+			raise NegotiationError('calculated m_%s differs from received m_%s' %
+				(i_o, i_o))
 
 		if i_o == 'a' and self.sas_algs == 'sas28x5':
 			# we don't need to calculate this if there's a verified retained secret
@@ -387,22 +411,21 @@ class EncryptedStanzaSession(StanzaSession):
 			parsed = xmpp.Node(node='<node>' + plaintext + '</node>')
 
 			if self.negotiated['recv_pubkey'] == 'hash':
-				fingerprint = parsed.getTagData('fingerprint')
-
-				# XXX find stored pubkey or terminate session
-				raise 'unimplemented'
+				# fingerprint = parsed.getTagData('fingerprint')
+				# FIXME find stored pubkey or terminate session
+				raise NotImplementedError()
 			else:
 				if self.negotiated['sign_algs'] == (XmlDsig + 'rsa-sha256'):
 					keyvalue = parsed.getTag(name='RSAKeyValue', namespace=XmlDsig)
 
-					n, e = map(lambda x: crypto.decode_mpi(base64.b64decode(
-						keyvalue.getTagData(x))), ('Modulus', 'Exponent'))
+					n, e = (crypto.decode_mpi(base64.b64decode(
+						keyvalue.getTagData(x))) for x in ('Modulus', 'Exponent'))
 					eir_pubkey = RSA.construct((n,long(e)))
 
-					pubkey_o = xmpp.c14n.c14n(keyvalue)
+					pubkey_o = xmpp.c14n.c14n(keyvalue, self._is_buggy_gajim())
 				else:
-					# XXX DSA, etc.
-					raise 'unimplemented'
+					# FIXME DSA, etc.
+					raise NotImplementedError()
 
 			enc_sig = parsed.getTag(name='SignatureValue',
 				namespace=XmlDsig).getData()
@@ -425,13 +448,14 @@ class EncryptedStanzaSession(StanzaSession):
 		mac_o_calculated = self.hmac(self.ks_o, content)
 
 		if self.negotiated['recv_pubkey']:
-			hash = crypto.sha256(mac_o_calculated)
+			hash_ = crypto.sha256(mac_o_calculated)
 
-			if not eir_pubkey.verify(hash, signature):
-				raise exceptions.NegotiationError, 'public key signature verification failed!'
+			if not eir_pubkey.verify(hash_, signature):
+				raise NegotiationError('public key signature verification failed!')
 
 		elif mac_o_calculated != mac_o:
-			raise exceptions.NegotiationError, 'calculated mac_%s differs from received mac_%s' % (i_o, i_o)
+			raise NegotiationError('calculated mac_%s differs from received mac_%s'
+				% (i_o, i_o))
 
 	def make_identity(self, form, dh_i):
 		if self.negotiated['send_pubkey']:
@@ -439,8 +463,8 @@ class EncryptedStanzaSession(StanzaSession):
 				pubkey = secrets.secrets().my_pubkey(self.conn.name)
 				fields = (pubkey.n, pubkey.e)
 
-				cb_fields = map(lambda f: base64.b64encode(crypto.encode_mpi(f)),
-					fields)
+				cb_fields = [base64.b64encode(crypto.encode_mpi(f)) for f in
+					fields]
 
 				pubkey_s = '<RSAKeyValue xmlns="http://www.w3.org/2000/09/xmldsig#"'
 				'><Modulus>%s</Modulus><Exponent>%s</Exponent></RSAKeyValue>' % \
@@ -448,7 +472,8 @@ class EncryptedStanzaSession(StanzaSession):
 		else:
 			pubkey_s = ''
 
-		form_s2 = ''.join(map(lambda el: xmpp.c14n.c14n(el), form.getChildren()))
+		form_s2 = ''.join(xmpp.c14n.c14n(el, self._is_buggy_gajim()) for el in \
+			form.getChildren())
 
 		old_c_s = self.c_s
 		content = self.n_o + self.n_s + crypto.encode_mpi(dh_i) + pubkey_s + \
@@ -478,8 +503,8 @@ class EncryptedStanzaSession(StanzaSession):
 			self.sas = crypto.sas_28x5(m_s, self.form_o)
 
 			if self.sigmai:
-				# XXX save retained secret?
-				self.check_identity(lambda : ())
+				# FIXME save retained secret?
+				self.check_identity(tuple)
 
 		return (xmpp.DataField(name='identity', value=base64.b64encode(id_s)),
 			xmpp.DataField(name='mac', value=base64.b64encode(m_s)))
@@ -521,7 +546,7 @@ class EncryptedStanzaSession(StanzaSession):
 		x.addChild(node=xmpp.DataField(name='init_pubkey', options=['none', 'key',
 			'hash'], typ='list-single'))
 
-		# XXX store key, use hash
+		# FIXME store key, use hash
 		x.addChild(node=xmpp.DataField(name='resp_pubkey', options=['none',
 			'key'], typ='list-single'))
 
@@ -544,12 +569,13 @@ class EncryptedStanzaSession(StanzaSession):
 			',') ]
 
 		x.addChild(node=xmpp.DataField(name='modp', typ='list-single',
-			options=map(lambda x: [ None, x ], modp_options)))
+			options=[[None, y] for y in modp_options]))
 
 		x.addChild(node=self.make_dhfield(modp_options, sigmai))
 		self.sigmai = sigmai
 
-		self.form_s = ''.join(map(lambda el: xmpp.c14n.c14n(el), x.getChildren()))
+		self.form_s = ''.join(xmpp.c14n.c14n(el, self._is_buggy_gajim()) for el \
+			in x.getChildren())
 
 		feature.addChild(node=x)
 
@@ -557,8 +583,8 @@ class EncryptedStanzaSession(StanzaSession):
 
 		self.send(request)
 
-	# 4.3 esession response (bob)
 	def verify_options_bob(self, form):
+		''' 4.3 esession response (bob) '''
 		negotiated = {'recv_pubkey': None, 'send_pubkey': None}
 		not_acceptable = []
 		ask_user = {}
@@ -572,12 +598,12 @@ class EncryptedStanzaSession(StanzaSession):
 
 		self.sas_algs = 'sas28x5'
 		self.cipher = AES
-		self.hash_alg = SHA256
+		self.hash_alg = sha256
 		self.compression = None
 
-		for name, field in map(lambda name: (name, form.getField(name)),
-		form.asDict().keys()):
-			options = map(lambda x: x[1], field.getOptions())
+		for name in form.asDict():
+			field = form.getField(name)
+			options = [x[1] for x in field.getOptions()]
 			values = field.getValues()
 
 			if not field.getType() in ('list-single', 'list-multi'):
@@ -620,14 +646,14 @@ class EncryptedStanzaSession(StanzaSession):
 				if (XmlDsig + 'rsa-sha256') in options:
 					negotiated['sign_algs'] = XmlDsig + 'rsa-sha256'
 			else:
-				# XXX some things are handled elsewhere, some things are
+				# FIXME some things are handled elsewhere, some things are
 				# not-implemented
 				pass
 
 		return (negotiated, not_acceptable, ask_user)
 
-	# 4.3 esession response (bob)
 	def respond_e2e_bob(self, form, negotiated, not_acceptable):
+		''' 4.3 esession response (bob) '''
 		response = xmpp.Message()
 		feature = response.NT.feature
 		feature.setNamespace(xmpp.NS_FEATURE)
@@ -678,9 +704,10 @@ class EncryptedStanzaSession(StanzaSession):
 			b64ed = base64.b64encode(to_add[name])
 			x.addChild(node=xmpp.DataField(name=name, value=b64ed))
 
-		self.form_o = ''.join(map(lambda el: xmpp.c14n.c14n(el),
-			form.getChildren()))
-		self.form_s = ''.join(map(lambda el: xmpp.c14n.c14n(el), x.getChildren()))
+		self.form_o = ''.join(xmpp.c14n.c14n(el, self._is_buggy_gajim()) for el \
+			in form.getChildren())
+		self.form_s = ''.join(xmpp.c14n.c14n(el, self._is_buggy_gajim()) for el \
+			in x.getChildren())
 
 		self.status = 'responded-e2e'
 
@@ -700,8 +727,8 @@ class EncryptedStanzaSession(StanzaSession):
 
 		self.send(response)
 
-	# 'Alice Accepts'
 	def verify_options_alice(self, form):
+		''' 'Alice Accepts' '''
 		negotiated = {}
 		ask_user = {}
 		not_acceptable = []
@@ -728,12 +755,12 @@ class EncryptedStanzaSession(StanzaSession):
 
 		return (negotiated, not_acceptable, ask_user)
 
-	# 'Alice Accepts', continued
 	def accept_e2e_alice(self, form, negotiated):
+		''' 'Alice Accepts', continued '''
 		self.encryptable_stanzas = ['message']
 		self.sas_algs = 'sas28x5'
 		self.cipher = AES
-		self.hash_alg = SHA256
+		self.hash_alg = sha256 
 		self.compression = None
 
 		self.negotiated = negotiated
@@ -746,7 +773,6 @@ class EncryptedStanzaSession(StanzaSession):
 
 		self.c_s = crypto.decode_mpi(base64.b64decode(form['counter']))
 		self.c_o = self.c_s ^ (2 ** (self.n - 1))
-
 		self.n_o = base64.b64decode(form['my_nonce'])
 
 		mod_p = int(form['modp'])
@@ -755,7 +781,6 @@ class EncryptedStanzaSession(StanzaSession):
 		e = self.es[mod_p]
 
 		self.d = crypto.decode_mpi(base64.b64decode(form['dhkeys']))
-
 		self.k = self.get_shared_secret(self.d, x, p)
 
 		result.addChild(node=xmpp.DataField(name='FORM_TYPE',
@@ -772,11 +797,11 @@ class EncryptedStanzaSession(StanzaSession):
 		else:
 			srses = secrets.secrets().retained_secrets(self.conn.name,
 				self.jid.getStripped())
-			rshashes = [self.hmac(self.n_s, rs) for (rs,v) in srses]
+			rshashes = [self.hmac(self.n_s, rs[0]) for rs in srses]
 
 			if not rshashes:
 				# we've never spoken before, but we'll pretend we have
-				rshash_size = self.hash_alg.digest_size
+				rshash_size = self.hash_alg().digest_size
 				rshashes.append(crypto.random_bytes(rshash_size))
 
 			rshashes = [base64.b64encode(rshash) for rshash in rshashes]
@@ -784,8 +809,8 @@ class EncryptedStanzaSession(StanzaSession):
 			result.addChild(node=xmpp.DataField(name='dhkeys',
 				value=base64.b64encode(crypto.encode_mpi(e))))
 
-			self.form_o = ''.join(map(lambda el: xmpp.c14n.c14n(el),
-				form.getChildren()))
+			self.form_o = ''.join(xmpp.c14n.c14n(el, self._is_buggy_gajim()) for \
+				el in form.getChildren())
 
 		# MUST securely destroy K unless it will be used later to generate the
 		# final shared secret
@@ -802,8 +827,8 @@ class EncryptedStanzaSession(StanzaSession):
 		else:
 			self.status = 'identified-alice'
 
-	# 4.5 esession accept (bob)
 	def accept_e2e_bob(self, form):
+		''' 4.5 esession accept (bob) '''
 		response = xmpp.Message()
 
 		init = response.NT.init
@@ -812,6 +837,7 @@ class EncryptedStanzaSession(StanzaSession):
 		x = xmpp.DataForm(typ='result')
 
 		for field in ('nonce', 'dhkeys', 'rshashes', 'identity', 'mac'):
+			# FIXME: will do nothing in real world...
 			assert field in form.asDict(), "alice's form didn't have a %s field" \
 				% field
 
@@ -820,18 +846,15 @@ class EncryptedStanzaSession(StanzaSession):
 		p = dh.primes[self.modp]
 
 		if crypto.sha256(crypto.encode_mpi(e)) != self.negotiated['He']:
-			raise exceptions.NegotiationError, 'SHA256(e) != He'
+			raise NegotiationError('SHA256(e) != He')
 
 		k = self.get_shared_secret(e, self.y, p)
-
 		self.kc_o, self.km_o, self.ks_o = self.generate_initiator_keys(k)
 
 		# 4.5.2 verifying alice's identity
-
 		self.verify_identity(form, e, False, 'a')
 
 		# 4.5.4 generating bob's final session keys
-
 		srs = ''
 
 		srses = secrets.secrets().retained_secrets(self.conn.name,
@@ -839,7 +862,8 @@ class EncryptedStanzaSession(StanzaSession):
 		rshashes = [base64.b64decode(rshash) for rshash in form.getField(
 			'rshashes').getValues()]
 
-		for (secret, verified) in srses:
+		for s in srses:
+			secret = s[0]
 			if self.hmac(self.n_o, secret) in rshashes:
 				srs = secret
 				break
@@ -893,7 +917,8 @@ class EncryptedStanzaSession(StanzaSession):
 		except IndexError:
 			return
 
-		for (secret, verified) in srses:
+		for s in srses:
+			secret = s[0]
 			if self.hmac(secret, 'Shared Retained Secret') == srshash:
 				srs = secret
 				break
@@ -909,7 +934,6 @@ class EncryptedStanzaSession(StanzaSession):
 		self.kc_o, self.km_o, self.ks_o = self.generate_responder_keys(k)
 
 		# 4.6.2 Verifying Bob's Identity
-
 		self.verify_identity(form, self.d, False, 'b')
 		# Note: If Alice discovers an error then she SHOULD ignore any encrypted
 		# content she received in the stanza.
@@ -924,10 +948,11 @@ class EncryptedStanzaSession(StanzaSession):
 			self.control.print_esession_details()
 
 	def do_retained_secret(self, k, old_srs):
-		'''calculate the new retained secret. determine if the user needs to check
-		the remote party's identity. set up callbacks for when the identity has
-		been verified.'''
-
+		'''
+		Calculate the new retained secret. determine if the user needs to check
+		the remote party's identity. Set up callbacks for when the identity has
+		been verified.
+		'''
 		new_srs = self.hmac(k, 'New Retained Secret')
 		self.srs = new_srs
 
@@ -967,7 +992,7 @@ class EncryptedStanzaSession(StanzaSession):
 
 			x = crypto.srand(2 ** (2 * self.n - 1), p - 1)
 
-			# XXX this may be a source of performance issues
+			# FIXME this may be a source of performance issues
 			e = crypto.powmod(g, x, p)
 
 			self.xes[modp] = x
@@ -985,21 +1010,19 @@ class EncryptedStanzaSession(StanzaSession):
 
 	def terminate_e2e(self):
 		self.terminate()
-
 		self.enable_encryption = False
 
 	def acknowledge_termination(self):
 		StanzaSession.acknowledge_termination(self)
-
 		self.enable_encryption = False
 
-	def fail_bad_negotiation(self, reason, fields = None):
-		'''sends an error and cancels everything.
-
-if fields is None, the remote party has given us a bad cryptographic value of some kind
-
-otherwise, list the fields we haven't implemented'''
+	def fail_bad_negotiation(self, reason, fields=None):
+		'''
+		Sends an error and cancels everything.
 
+		If fields is None, the remote party has given us a bad cryptographic
+		value of some kind. Otherwise, list the fields we haven't implemented
+		'''
 		err = xmpp.Error(xmpp.Message(), xmpp.ERR_FEATURE_NOT_IMPLEMENTED)
 		err.T.error.T.text.setData(reason)
 
@@ -1025,7 +1048,6 @@ otherwise, list the fields we haven't implemented'''
 	def cancelled_negotiation(self):
 		StanzaSession.cancelled_negotiation(self)
 		self.enable_encryption = False
-
 		self.km_o = ''
 
 # vim: se ts=3:
