@@ -77,7 +77,7 @@ def get_proxy_data_from_dict(proxy):
 		# with proxy!=bosh or with bosh over HTTP proxy we're connecting to proxy
 		# machine
 		tcp_host, tcp_port = proxy['host'], proxy['port']
-	if proxy['useauth']:
+	if proxy.get('useauth', False):
 		proxy_user, proxy_pass = proxy['user'], proxy['pass']
 	return tcp_host, tcp_port, proxy_user, proxy_pass
 
@@ -221,7 +221,7 @@ class NonBlockingTransport(PlugIn):
 			if hasattr(self, '_owner') and hasattr(self._owner, 'Dispatcher'):
 				self.on_receive = self._owner.Dispatcher.ProcessNonBlocking
 			else:
-				log.warning('No Dispatcher plugged. Received data will not be processed')
+				log.info('No Dispatcher plugged. Received data will not be processed')
 				self.on_receive = None
 			return
 		self.on_receive = recv_handler
@@ -441,7 +441,7 @@ class NonBlockingTCP(NonBlockingTransport, IdleObject):
 			self._sock.shutdown(socket.SHUT_RDWR)
 			self._sock.close()
 		except socket.error, (errnum, errstr):
-			log.error('Error while disconnecting socket: %s' % errstr)
+			log.info('Error while disconnecting socket: %s' % errstr)
 		self.fd = -1
 		NonBlockingTransport.disconnect(self, do_callback)
 
