@@ -137,8 +137,7 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 	# END __init__
 
 	def dispatch(self, event, data):
-		if event in gajim.handlers:
-			gajim.handlers[event](self.name, data)
+		gajim.interface.dispatch(event, self.name, data)
 
 	def _reconnect(self):
 		# Do not try to reco while we are already trying
@@ -365,7 +364,7 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 	def send_message(self, jid, msg, keyID, type_='chat', subject='',
 	chatstate=None, msg_id=None, composing_xep=None, resource=None,
 	user_nick=None, xhtml=None, session=None, forward_from=None, form_node=None,
-	original_message=None, delayed=None):
+	original_message=None, delayed=None, callback=None, callback_args=[]):
 		fjid = jid
 
 		if msg and not xhtml and gajim.config.get(
@@ -378,7 +377,9 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 			return
 
 		if self.status in ('invisible', 'offline'):
-			self.dispatch('MSGERROR', [unicode(jid), '-1', _('You are not connected or not visible to others. Your message could not be sent.'), None, None])
+			self.dispatch('MSGERROR', [unicode(jid), -1,
+				 _('You are not connected or not visible to others. Your message '
+				'could not be sent.'), None, None, session])
 			return
 
 		msgtxt = msg
@@ -401,7 +402,7 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 				# Encryption failed, do not send message
 				tim = time.localtime()
 				self.dispatch('MSGNOTSENT', (jid, error, msgtxt, tim, session))
-				return 3
+				return
 
 		if type_ == 'chat':
 			msg_iq = common.xmpp.Message(to=fjid, body=msgtxt, typ=type_,
@@ -457,7 +458,7 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 			if session.enable_encryption:
 				msg_iq = session.encrypt_stanza(msg_iq)
 
-		def on_send_ok():
+		def on_send_ok(id):
 			no_log_for = gajim.config.get_per('accounts', self.name, 'no_log_for')
 			ji = gajim.get_jid_without_resource(jid)
 			if session.is_loggable() and self.name not in no_log_for and\
@@ -475,16 +476,18 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 
 			self.dispatch('MSGSENT', (jid, msg, keyID))
 
+			if callback:
+				callback(id, *callback_args)
+
 		def on_send_not_ok(reason):
 			reason += ' ' + _('Your message could not be sent.')
-			self.dispatch('MSGERROR', [jid, '-1', reason, None, None, session])
-
+			self.dispatch('MSGERROR', [jid, -1, reason, None, None, session])
 		ret = self.connection.send(msg_iq, msg is not None, on_ok=on_send_ok,
 			on_not_ok=on_send_not_ok)
 		if ret == -1:
 			# Contact Offline
-			self.dispatch('MSGERROR', [jid, '-1', _('Contact is offline. Your message could not be sent.'), None, None, session])
-		return ret
+			self.dispatch('MSGERROR', [jid, -1, _('Contact is offline. Your message could not be sent.'), None, None, session])
+
 
 	def send_stanza(self, stanza):
 		# send a stanza untouched
@@ -521,6 +524,11 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 			self.connection.getRoster().setItem(jid = jid, name = name,
 				groups = groups)
 
+	def update_contacts(self, contacts):
+		'''update multiple roster items'''
+		if self.connection:
+			self.connection.getRoster().setItemMulti(contacts)
+
 	def new_account(self, name, config, sync = False):
 		gajim.log.debug('This should not happen (new_account)')
 
@@ -573,10 +581,20 @@ class ConnectionZeroconf(ConnectionHandlersZeroconf):
 
 	def _event_dispatcher(self, realm, event, data):
 		if realm == '':
-			if event == common.xmpp.transports.DATA_RECEIVED:
+			if event == common.xmpp.transports_nb.DATA_RECEIVED:
 				self.dispatch('STANZA_ARRIVED', unicode(data, errors = 'ignore'))
-			elif event == common.xmpp.transports.DATA_SENT:
+			elif event == common.xmpp.transports_nb.DATA_SENT:
 				self.dispatch('STANZA_SENT', unicode(data))
+			elif event == common.xmpp.transports.DATA_ERROR:
+				thread_id = data[1]
+				frm = unicode(data[0])
+				session = self.get_or_create_session(frm, thread_id)
+				self.dispatch('MSGERROR', [frm, -1,
+	            _('Connection to host could not be established: Timeout while '
+					'sending data.'), None, None, session])
+
+	def load_roster_from_db(self):
+		return
 
 # END ConnectionZeroconf
 
