@@ -1,23 +1,29 @@
-##	dataforms.py
+# -*- coding:utf-8 -*-
+## src/dataforms_widget.py
 ##
-## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <nkour@jabber.org>
-## Copyright (C) 2005 Dimitur Kirov <dkirov@gmail.com>
-## Copyright (C) 2003-2005 Vincent Hanquez <tab@snarc.org>
+## Copyright (C) 2003-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006 Tomasz Melcer <liori AT exroot.org>
+## Copyright (C) 2006-2007 Jean-Marie Traissard <jim AT lapin.org>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
-""" This module contains widget that can display data form (JEP-0004).
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
+
+''' This module contains widget that can display data form (JEP-0004).
 Words single and multiple refers here to types of data forms:
 single means these with one record of data (without <reported/> element),
-multiple - these which may contain more data (with <reported/> element)."""
+multiple - these which may contain more data (with <reported/> element).'''
 
 import gtk
 import gobject
@@ -32,14 +38,15 @@ import itertools
 
 class DataFormWidget(gtk.Alignment, object):
 # "public" interface
-	""" Data Form widget. Use like any other widget. """
+	''' Data Form widget. Use like any other widget. '''
 	def __init__(self, dataformnode=None):
-		""" Create a widget. """
+		''' Create a widget. '''
 		gtk.Alignment.__init__(self, xscale=1.0, yscale=1.0)
 
 		self._data_form = None
 
-		self.xml=gtkgui_helpers.get_glade('data_form_window.glade', 'data_form_vbox')
+		self.xml = gtkgui_helpers.get_glade('data_form_window.glade',
+			'data_form_vbox')
 		self.xml.signal_autoconnect(self)
 		for name in ('instructions_label', 'instructions_hseparator',
 				'single_form_viewport', 'data_form_types_notebook',
@@ -58,7 +65,7 @@ class DataFormWidget(gtk.Alignment, object):
 		selection.set_mode(gtk.SELECTION_MULTIPLE)
 
 	def set_data_form(self, dataform):
-		""" Set the data form (xmpp.DataForm) displayed in widget. """
+		''' Set the data form (xmpp.DataForm) displayed in widget. '''
 		assert isinstance(dataform, dataforms.DataForm)
 
 		self.del_data_form()
@@ -69,14 +76,15 @@ class DataFormWidget(gtk.Alignment, object):
 			self.build_multiple_data_form()
 
 		# create appropriate description for instructions field if there isn't any
-		if dataform.instructions=='':
+		if dataform.instructions == '':
 			self.instructions_label.set_no_show_all(True)
 			self.instructions_label.hide()
 		else:
 			self.instructions_label.set_text(dataform.instructions)
+			gtkgui_helpers.label_set_autowrap(self.instructions_label)
 
 	def get_data_form(self):
-		""" Data form displayed in the widget or None if no form. """
+		''' Data form displayed in the widget or None if no form. '''
 		return self._data_form
 
 	def del_data_form(self):
@@ -84,26 +92,26 @@ class DataFormWidget(gtk.Alignment, object):
 		self._data_form = None
 
 	data_form = property(get_data_form, set_data_form, del_data_form,
-		"Data form presented in a widget")
+		'Data form presented in a widget')
 
 	def get_title(self):
-		""" Get the title of data form, as a unicode object. If no
-		title or no form, returns u''. Useful for setting window title. """
+		''' Get the title of data form, as a unicode object. If no
+		title or no form, returns u''. Useful for setting window title. '''
 		if self._data_form is not None:
 			if self._data_form.title is not None:
 				return self._data_form.title
 		return u''
 
-	title = property(get_title, None, None, "Data form title")
+	title = property(get_title, None, None, 'Data form title')
 
 	def show(self):
-		""" Treat 'us' as one widget. """
+		''' Treat 'us' as one widget. '''
 		self.show_all()
 
 # "private" methods
 
 # we have actually two different kinds of data forms: one is a simple form to fill,
-# second is a table with several records; 
+# second is a table with several records;
 
 	def empty_method(self):
 		pass
@@ -144,23 +152,27 @@ class DataFormWidget(gtk.Alignment, object):
 
 		# creating model for form...
 		fieldtypes = []
+		fieldvars = []
 		for field in self._data_form.reported.iter_fields():
 			# note: we store also text-private and hidden fields,
 			# we just do not display them.
 			# TODO: boolean fields
 			#elif field.type=='boolean': fieldtypes.append(bool)
 			fieldtypes.append(str)
+			fieldvars.append(field.var)
 
 		self.multiplemodel = gtk.ListStore(*fieldtypes)
 
 		# moving all data to model
 		for item in self._data_form.iter_records():
-			# TODO: probably wrong... (.value[s]?, fields not in the same order?)
-			# not checking multiple-item forms...
-			self.multiplemodel.append([field.value for field in item.iter_fields()])
+			iter_ = self.multiplemodel.append()
+			for field in item.iter_fields():
+				self.multiplemodel.set_value(iter_, fieldvars.index(field.var),
+					field.value)
 
 		# constructing columns...
-		for field, counter in zip(self._data_form.reported.iter_fields(), itertools.count()):
+		for field, counter in zip(self._data_form.reported.iter_fields(),
+		itertools.count()):
 			self.records_treeview.append_column(
 				gtk.TreeViewColumn(field.label,	gtk.CellRendererText(),
 					text=counter))
@@ -203,8 +215,8 @@ class DataFormWidget(gtk.Alignment, object):
 			self.remove_button.set_sensitive(True)
 			self.edit_button.set_sensitive(True)
 			_, (path,) = selection.get_selected_rows()
-			iter = model.get_iter(path)
-			if model.iter_next(iter) is None:
+			iter_ = model.get_iter(path)
+			if model.iter_next(iter_) is None:
 				self.up_button.set_sensitive(True)
 				self.down_button.set_sensitive(False)
 			elif path == (0, ):
@@ -229,29 +241,31 @@ class DataFormWidget(gtk.Alignment, object):
 
 	def on_remove_button_clicked(self, widget):
 		selection = self.records_treeview.get_selection()
-		model, rowrefs = selection.get_selected_rows()	# rowref is a list of paths
+		model, rowrefs = selection.get_selected_rows()
+		# rowref is a list of paths
 		for i in xrange(len(rowrefs)):
 			rowrefs[i] = gtk.TreeRowReference(model, rowrefs[i])
-		# rowref is a list of row references; need to convert because we will modify the model,
-		# paths would change
+		# rowref is a list of row references; need to convert because we will
+		# modify the model, paths would change
 		for rowref in rowrefs:
 			del model[rowref.get_path()]
-	
+
 	def on_up_button_clicked(self, widget):
 		selection = self.records_treeview.get_selection()
 		model, (path,) = selection.get_selected_rows()
-		iter = model.get_iter(path)
-		previter = model.get_iter((path[0]-1,))	# constructing path for previous iter
-		model.swap(iter, previter)
+		iter_ = model.get_iter(path)
+		# constructing path for previous iter
+		previter = model.get_iter((path[0]-1,))
+		model.swap(iter_, previter)
 
 		self.refresh_multiple_buttons()
 
 	def on_down_button_clicked(self, widget):
 		selection = self.records_treeview.get_selection()
 		model, (path,) = selection.get_selected_rows()
-		iter = model.get_iter(path)
-		nextiter = model.iter_next(iter)
-		model.swap(iter, nextiter)
+		iter_ = model.get_iter(path)
+		nextiter = model.iter_next(iter_)
+		model.swap(iter_, nextiter)
 
 		self.refresh_multiple_buttons()
 
@@ -259,9 +273,9 @@ class DataFormWidget(gtk.Alignment, object):
 		self.refresh_multiple_buttons()
 
 class SingleForm(gtk.Table, object):
-	""" Widget that represent DATAFORM_SINGLE mode form. Because this is used
+	''' Widget that represent DATAFORM_SINGLE mode form. Because this is used
 	not only to display single forms, but to form input windows of multiple-type
-	forms, it is in another class."""
+	forms, it is in another class.'''
 	def __init__(self, dataform):
 		assert isinstance(dataform, dataforms.SimpleDataForm)
 
@@ -269,18 +283,16 @@ class SingleForm(gtk.Table, object):
 		self.set_col_spacings(12)
 		self.set_row_spacings(6)
 
-		self.tooltips = gtk.Tooltips()
-
 		def decorate_with_tooltip(widget, field):
-			""" Adds a tooltip containing field's description to a widget.
+			''' Adds a tooltip containing field's description to a widget.
 			Creates EventBox if widget doesn't have its own gdk window.
-			Returns decorated widget. """
-			if field.description!='':
-				if widget.flags()&gtk.NO_WINDOW:
+			Returns decorated widget. '''
+			if field.description != '':
+				if widget.flags() & gtk.NO_WINDOW:
 					evbox = gtk.EventBox()
 					evbox.add(widget)
 					widget = evbox
-				self.tooltips.set_tip(widget, field.description)
+				widget.set_tooltip_text(field.description)
 			return widget
 
 		self._data_form = dataform
@@ -293,63 +305,95 @@ class SingleForm(gtk.Table, object):
 
 		# for each field...
 		for field in self._data_form.iter_fields():
-			if field.type=='hidden': continue
+			if field.type == 'hidden': continue
 
 			commonlabel = True
 			commonlabelcenter = False
 			commonwidget = True
 			widget = None
 
-			if field.type=='boolean':
+			if field.type == 'boolean':
 				commonlabelcenter = True
 				widget = gtk.CheckButton()
-				widget.connect('toggled', self.on_boolean_checkbutton_toggled, field)
+				widget.connect('toggled', self.on_boolean_checkbutton_toggled,
+					field)
 				widget.set_active(field.value)
 
-			elif field.type=='fixed':
+			elif field.type == 'fixed':
 				leftattach = 1
 				rightattach = 2
 				if field.label is None:
 					commonlabel = False
 					leftattach = 0
-				
-				commonwidget=False
+
+				commonwidget = False
 				widget = gtk.Label(field.value)
 				widget.set_line_wrap(True)
-				self.attach(widget, leftattach, rightattach, linecounter, linecounter+1,
-					xoptions=gtk.FILL, yoptions=gtk.FILL)
+				self.attach(widget, leftattach, rightattach, linecounter,
+					linecounter+1, xoptions=gtk.FILL, yoptions=gtk.FILL)
 
 			elif field.type == 'list-single':
-				# TODO: When more than few choices, make a list
-				# TODO: Think of moving that to another function (it could be used
-				# TODO: in stage2 of adhoc commands too).
 				# TODO: What if we have radio buttons and non-required field?
 				# TODO: We cannot deactivate them all...
-				widget = gtk.VBox()
-				first_radio = None
-				for value, label in field.iter_options():
-					radio = gtk.RadioButton(first_radio, label=label)
-					radio.connect('toggled', self.on_list_single_radiobutton_toggled,
-						field, value)
-					if first_radio is None:
-						first_radio = radio
-						if field.value=='':	# TODO: is None when done
-							field.value = value
-					if value == field.value:
-						radio.set_active(True)
-					widget.set_sensitive(readwrite)
-					widget.pack_start(radio, expand=False)
+				if len(field.options) < 6:
+					# 5 option max: show radiobutton
+					widget = gtk.VBox()
+					first_radio = None
+					for value, label in field.iter_options():
+						if not label:
+							label = value
+						radio = gtk.RadioButton(first_radio, label=label)
+						radio.connect('toggled',
+							self.on_list_single_radiobutton_toggled, field, value)
+						if first_radio is None:
+							first_radio = radio
+							if field.value == '':	# TODO: is None when done
+								field.value = value
+						if value == field.value:
+							radio.set_active(True)
+						widget.pack_start(radio, expand=False)
+				else:
+					# more than 5 options: show combobox
+					def on_list_single_combobox_changed(combobox, f):
+						iter_ = combobox.get_active_iter()
+						if iter_:
+							model = combobox.get_model()
+							f.value = model[iter_][1]
+						else:
+							f.value = ''
+					widget = gtkgui_helpers.create_combobox(field.options,
+						field.value)
+					widget.connect('changed', on_list_single_combobox_changed, field)
+				widget.set_sensitive(readwrite)
 
 			elif field.type == 'list-multi':
 				# TODO: When more than few choices, make a list
-				widget = gtk.VBox()
-				for value, label in field.iter_options():
-					check = gtk.CheckButton(label, use_underline=False)
-					check.set_active(value in field.values)
-					check.connect('toggled', self.on_list_multi_checkbutton_toggled,
-						field, value)
-					widget.set_sensitive(readwrite)
+				if len(field.options) < 6:
+					# 5 option max: show checkbutton
+					widget = gtk.VBox()
+					for value, label in field.iter_options():
+						check = gtk.CheckButton(label, use_underline=False)
+						check.set_active(value in field.values)
+						check.connect('toggled',
+							self.on_list_multi_checkbutton_toggled, field, value)
 					widget.pack_start(check, expand=False)
+				else:
+					# more than 5 options: show combobox
+					def on_list_multi_treeview_changed(selection, f):
+						def for_selected(treemodel, path, iter):
+							vals.append(treemodel[iter][1])
+						vals = []
+						selection.selected_foreach(for_selected)
+						field.values = vals[:]
+					widget = gtk.ScrolledWindow()
+					widget.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+					tv = gtkgui_helpers.create_list_multi(field.options,
+						field.values)
+					widget.add(tv)
+					widget.set_size_request(-1, 120)
+					tv.get_selection().connect('changed',
+						on_list_multi_treeview_changed, field)
+				widget.set_sensitive(readwrite)
 
 			elif field.type == 'jid-single':
 				widget = gtk.Entry()
@@ -418,10 +462,10 @@ class SingleForm(gtk.Table, object):
 
 				textwidget = gtk.TextView()
 				textwidget.set_wrap_mode(gtk.WRAP_WORD)
-				textwidget.get_buffer().connect('changed', self.on_text_multi_textbuffer_changed,
-					field)
+				textwidget.get_buffer().connect('changed',
+					self.on_text_multi_textbuffer_changed, field)
 				textwidget.get_buffer().set_text(field.value)
-				
+
 				widget = gtk.ScrolledWindow()
 				widget.add(textwidget)
 
@@ -429,13 +473,15 @@ class SingleForm(gtk.Table, object):
 				widget=decorate_with_tooltip(widget, field)
 				self.attach(widget, 1, 2, linecounter, linecounter+1)
 
-			else:# field.type == 'text-single' or field.type is nonstandard:
+			else:
+				# field.type == 'text-single' or field.type is nonstandard:
 				# JEP says that if we don't understand some type, we
 				# should handle it as text-single
 				commonlabelcenter = True
 				if readwrite:
 					widget = gtk.Entry()
-					widget.connect('changed', self.on_text_single_entry_changed, field)
+					widget.connect('changed', self.on_text_single_entry_changed,
+						field)
 					widget.set_sensitive(readwrite)
 					if field.value is None:
 						field.value = u''
@@ -519,30 +565,30 @@ class SingleForm(gtk.Table, object):
 		field.values = values
 
 	def on_jid_multi_add_button_clicked(self, widget, treeview, model, field):
-		jid = 'new@jabber.id'
+		#Default jid
+		jid = _('new@jabber.id')
 		if jid in field.values:
 			i = 1
-			while 'new%d@jabber.id' % i in field.values:
+			while _('new%d@jabber.id') % i in field.values:
 				i += 1
-			jid = 'new%d@jabber.id' % i
-		iter = model.insert(999999, (jid,))
-		treeview.set_cursor(model.get_path(iter), treeview.get_column(0), True)
+			jid = _('new%d@jabber.id') % i
+		iter_ = model.insert(999999, (jid,))
+		treeview.set_cursor(model.get_path(iter_), treeview.get_column(0), True)
 		field.values = field.values + [jid]
 
 	def on_jid_multi_edit_button_clicked(self, widget, treeview):
-		model, iter = treeview.get_selection().get_selected()
-		assert iter is not None
+		model, iter_ = treeview.get_selection().get_selected()
+		assert iter_ is not None
 
-		treeview.set_cursor(model.get_path(iter), treeview.get_column(0), True)
+		treeview.set_cursor(model.get_path(iter_), treeview.get_column(0), True)
 
 	def on_jid_multi_remove_button_clicked(self, widget, treeview, field):
 		selection = treeview.get_selection()
-		model = treeview.get_model()
 		deleted = []
 
-		def remove(model, path, iter, deleted):
-			deleted+=model[iter]
-			model.remove(iter)
+		def remove(model, path, iter_, deleted):
+			deleted+=model[iter_]
+			model.remove(iter_)
 
 		selection.selected_foreach(remove, deleted)
 		field.values = (v for v in field.values if v not in deleted)
@@ -550,3 +596,5 @@ class SingleForm(gtk.Table, object):
 	def on_jid_multi_clean_button_clicked(self, widget, model, field):
 		model.clear()
 		del field.values
+
+# vim: se ts=3:
