@@ -34,10 +34,10 @@
 # - def update_actions(self)
 # - def default_action(self)
 # - def _find_item(self, jid, node)
-# - def _add_item(self, jid, node, item, force)
-# - def _update_item(self, iter, jid, node, item)
-# - def _update_info(self, iter, jid, node, identities, features, data)
-# - def _update_error(self, iter, jid, node)
+# - def _add_item(self, jid, node, parent_node, item, force)
+# - def _update_item(self, iter_, jid, node, item)
+# - def _update_info(self, iter_, jid, node, identities, features, data)
+# - def _update_error(self, iter_, jid, node)
 #
 # * Should call the super class for this method.
 # All others do not have to call back to the super class. (but can if they want
@@ -260,8 +260,8 @@ class ServicesCache:
 		# Grab the first identity with an icon
 		for identity in identities:
 			try:
-				cat, type = identity['category'], identity['type']
-				info = _agent_type_info[(cat, type)]
+				cat, type_ = identity['category'], identity['type']
+				info = _agent_type_info[(cat, type_)]
 			except KeyError:
 				continue
 			filename = info[1]
@@ -269,7 +269,6 @@ class ServicesCache:
 				break
 		else:
 			# Loop fell through, default to unknown
-			cat = type = 0
 			info = _agent_type_info[(0, 0)]
 			filename = info[1]
 		if not filename: # we don't have an image to show for this type
@@ -435,6 +434,7 @@ class ServiceDiscoveryWindow(object):
 		self.browser = None
 		self.children = []
 		self.dying = False
+		self.node = None
 
 		# Check connection
 		if gajim.connections[account].connected < 2:
@@ -524,7 +524,7 @@ _('Without a connection, you can not browse available services'))
 		bannerfont = gajim.config.get_per('themes', theme, 'bannerfont')
 		bannerfontattrs = gajim.config.get_per('themes', theme,
 			'bannerfontattrs')
-		
+
 		if bannerfont:
 			font = pango.FontDescription(bannerfont)
 		else:
@@ -535,10 +535,10 @@ _('Without a connection, you can not browse available services'))
 				font.set_weight(pango.WEIGHT_HEAVY)
 			if 'I' in bannerfontattrs:
 				font.set_style(pango.STYLE_ITALIC)
-		
+
 		font_attrs = 'font_desc="%s"' % font.to_string()
 		font_size = font.get_size()
-		
+
 		# in case there is no font specified we use x-large font size
 		if font_size == 0:
 			font_attrs = '%s size="large"' % font_attrs
@@ -548,7 +548,7 @@ _('Without a connection, you can not browse available services'))
 			markup = '%s\n<span font_desc="%s" size="small">%s</span>' % \
 									(markup, font.to_string(), text_after)
 		self.banner.set_markup(markup)
-	
+
 	def paint_banner(self):
 		'''Repaint the banner with theme color'''
 		theme = gajim.config.get('roster_theme')
@@ -561,7 +561,7 @@ _('Without a connection, you can not browse available services'))
 			default_bg = False
 		else:
 			default_bg = True
-		
+
 		if textcolor:
 			color = gtk.gdk.color_parse(textcolor)
 			self.banner.modify_fg(gtk.STATE_NORMAL, color)
@@ -572,22 +572,22 @@ _('Without a connection, you can not browse available services'))
 			self._on_style_set_event(self.banner, None, default_fg, default_bg)
 		if self.browser:
 			self.browser.update_theme()
-	
+
 	def disconnect_style_event(self):
 		if self.style_event_id:
 			self.banner.disconnect(self.style_event_id)
 			self.style_event_id = 0
-	
+
 	def connect_style_event(self, set_fg = False, set_bg = False):
 		self.disconnect_style_event()
-		self.style_event_id = self.banner.connect('style-set', 
+		self.style_event_id = self.banner.connect('style-set',
 					self._on_style_set_event, set_fg, set_bg)
-	
+
 	def _on_style_set_event(self, widget, style, *opts):
-		''' set style of widget from style class *.Frame.Eventbox 
+		''' set style of widget from style class *.Frame.Eventbox
 			opts[0] == True -> set fg color
 			opts[1] == True -> set bg color	'''
-		
+
 		self.disconnect_style_event()
 		if opts[1]:
 			bg_color = widget.style.bg[gtk.STATE_SELECTED]
@@ -597,7 +597,7 @@ _('Without a connection, you can not browse available services'))
 			self.banner.modify_fg(gtk.STATE_NORMAL, fg_color)
 		self.banner.ensure_style()
 		self.connect_style_event(opts[0], opts[1])
-	
+
 	def destroy(self, chain = False):
 		'''Close the browser. This can optionally close its children and
 		propagate to the parent. This should happen on actions like register,
@@ -811,7 +811,7 @@ class AgentBrowser:
 		if self.browse_button:
 			self.browse_button.destroy()
 			self.browse_button = None
-	
+
 	def _set_title(self, jid, node, identities, features, data):
 		'''Set the window title based on agent info.'''
 		# Set the banner and window title
@@ -868,12 +868,12 @@ class AgentBrowser:
 	def on_browse_button_clicked(self, widget = None):
 		'''When we want to browse an agent:
 		Open a new services window with a browser for the agent type.'''
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		if not iter:
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if not iter_:
 			return
-		jid = model[iter][0].decode('utf-8')
+		jid = model[iter_][0].decode('utf-8')
 		if jid:
-			node = model[iter][1].decode('utf-8')
+			node = model[iter_][1].decode('utf-8')
 			self.window.open(jid, node)
 
 	def update_actions(self):
@@ -881,11 +881,11 @@ class AgentBrowser:
 		activate action buttons based on the agent's info.'''
 		if self.browse_button:
 			self.browse_button.set_sensitive(False)
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		if not iter:
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if not iter_:
 			return
-		jid = model[iter][0].decode('utf-8')
-		node = model[iter][1].decode('utf-8')
+		jid = model[iter_][0].decode('utf-8')
+		node = model[iter_][1].decode('utf-8')
 		if jid:
 			self.cache.get_info(jid, node, self._update_actions, nofetch = True)
 
@@ -900,11 +900,11 @@ class AgentBrowser:
 	def default_action(self):
 		'''When we double-click a row:
 		perform the default action on the selected item.'''
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		if not iter:
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if not iter_:
 			return
-		jid = model[iter][0].decode('utf-8')
-		node = model[iter][1].decode('utf-8')
+		jid = model[iter_][0].decode('utf-8')
+		node = model[iter_][1].decode('utf-8')
 		if jid:
 			self.cache.get_info(jid, node, self._default_action, nofetch = True)
 
@@ -935,15 +935,15 @@ class AgentBrowser:
 	def _find_item(self, jid, node):
 		'''Check if an item is already in the treeview. Return an iter to it
 		if so, None otherwise.'''
-		iter = self.model.get_iter_root()
-		while iter:
-			cjid = self.model.get_value(iter, 0).decode('utf-8')
-			cnode = self.model.get_value(iter, 1).decode('utf-8')
+		iter_ = self.model.get_iter_root()
+		while iter_:
+			cjid = self.model.get_value(iter_, 0).decode('utf-8')
+			cnode = self.model.get_value(iter_, 1).decode('utf-8')
 			if jid == cjid and node == cnode:
 				break
-			iter = self.model.iter_next(iter)
-		if iter:
-			return iter
+			iter_ = self.model.iter_next(iter_)
+		if iter_:
+			return iter_
 		return None
 
 	def _agent_items(self, jid, node, items, force):
@@ -963,31 +963,30 @@ _('This service does not contain any items to browse.'))
 		# We got a list of items
 		self.window.services_treeview.set_model(None)
 		for item in items:
-			jid = item['jid']
-			node = item.get('node', '')
+			jid_ = item['jid']
+			node_ = item.get('node', '')
 			# If such an item is already here: don't add it
-			if self._find_item(jid, node):
+			if self._find_item(jid_, node_):
 				continue
 			self._total_items += 1
-			self._add_item(jid, node, item, force)
+			self._add_item(jid_, node_, node, item, force)
 		self.window.services_treeview.set_model(self.model)
 
 	def _agent_info(self, jid, node, identities, features, data):
 		'''Callback for when we receive info about an agent's item.'''
-		addr = get_agent_address(jid, node)
-		iter = self._find_item(jid, node)
-		if not iter:
+		iter_ = self._find_item(jid, node)
+		if not iter_:
 			# Not in the treeview, stop
 			return
 		if identities == 0:
 			# The server returned an error
-			self._update_error(iter, jid, node)
+			self._update_error(iter_, jid, node)
 		else:
 			# We got our info
-			self._update_info(iter, jid, node, identities, features, data)
+			self._update_info(iter_, jid, node, identities, features, data)
 		self.update_actions()
 
-	def _add_item(self, jid, node, item, force):
+	def _add_item(self, jid, node, parent_node, item, force):
 		'''Called when an item should be added to the model. The result of a
 		disco#items query.'''
 		self.model.append((jid, node, item.get('name', ''),
@@ -1104,15 +1103,15 @@ class ToplevelAgentBrowser(AgentBrowser):
 			if not props or self.tooltip.id != props[0]:
 				self.tooltip.hide_tooltip()
 		if props:
-			[row, col, x, y] = props
-			iter = None
+			row = props[0]
+			iter_ = None
 			try:
-				iter = self.model.get_iter(row)
+				iter_ = self.model.get_iter(row)
 			except Exception:
 				self.tooltip.hide_tooltip()
 				return
-			jid = self.model[iter][0]
-			state = self.model[iter][4]
+			jid = self.model[iter_][0]
+			state = self.model[iter_][4]
 			# Not a category, and we have something to say about state
 			if jid and state > 0 and \
 					(self.tooltip.timeout == 0 or self.tooltip.id != props[0]):
@@ -1138,11 +1137,11 @@ class ToplevelAgentBrowser(AgentBrowser):
 		# Icon Renderer
 		renderer = gtk.CellRendererPixbuf()
 		renderer.set_property('xpad', 6)
-		col.pack_start(renderer, expand = False)
+		col.pack_start(renderer, expand=False)
 		col.set_cell_data_func(renderer, self._pixbuf_renderer_data_func)
 		# Text Renderer
 		renderer = gtk.CellRendererText()
-		col.pack_start(renderer, expand = True)
+		col.pack_start(renderer, expand=True)
 		col.set_cell_data_func(renderer, self._text_renderer_data_func)
 		renderer.set_property('foreground', 'dark gray')
 		# Save this so we can go along with theme changes
@@ -1239,10 +1238,10 @@ class ToplevelAgentBrowser(AgentBrowser):
 	def on_search_button_clicked(self, widget = None):
 		'''When we want to search something:
 		open search window'''
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		if not iter:
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if not iter_:
 			return
-		service = model[iter][0].decode('utf-8')
+		service = model[iter_][0].decode('utf-8')
 		if service in gajim.interface.instances[self.account]['search']:
 			gajim.interface.instances[self.account]['search'][service].window.\
 				present()
@@ -1264,21 +1263,21 @@ class ToplevelAgentBrowser(AgentBrowser):
 	def on_execute_button_clicked(self, widget=None):
 		'''When we want to execute a command:
 		open adhoc command window'''
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		if not iter:
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if not iter_:
 			return
-		service = model[iter][0].decode('utf-8')
-		node = model[iter][1].decode('utf-8')
+		service = model[iter_][0].decode('utf-8')
+		node = model[iter_][1].decode('utf-8')
 		adhoc_commands.CommandWindow(self.account, service, commandnode=node)
 
 	def on_register_button_clicked(self, widget = None):
 		'''When we want to register an agent:
 		request information about registering with the agent and close the
 		window.'''
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		if not iter:
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if not iter_:
 			return
-		jid = model[iter][0].decode('utf-8')
+		jid = model[iter_][0].decode('utf-8')
 		if jid:
 			gajim.connections[self.account].request_register_agent_info(jid)
 			self.window.destroy(chain = True)
@@ -1286,10 +1285,10 @@ class ToplevelAgentBrowser(AgentBrowser):
 	def on_join_button_clicked(self, widget):
 		'''When we want to join an IRC room or create a new MUC room:
 		Opens the join_groupchat_window.'''
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		if not iter:
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if not iter_:
 			return
-		service = model[iter][0].decode('utf-8')
+		service = model[iter_][0].decode('utf-8')
 		if 'join_gc' not in gajim.interface.instances[self.account]:
 			try:
 				dialogs.JoinGroupchatWindow(self.account, service)
@@ -1310,25 +1309,25 @@ class ToplevelAgentBrowser(AgentBrowser):
 			self.join_button.set_sensitive(False)
 		if self.search_button:
 			self.search_button.set_sensitive(False)
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		if not iter:
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if not iter_:
 			return
-		if not model[iter][0]:
+		if not model[iter_][0]:
 			# We're on a category row
 			return
-		if model[iter][4] != 0:
+		if model[iter_][4] != 0:
 			# We don't have the info (yet)
 			# It's either unknown or a transport, register button should be active
 			if self.register_button:
 				self.register_button.set_sensitive(True)
 			# Guess what kind of service we're dealing with
 			if self.browse_button:
-				jid = model[iter][0].decode('utf-8')
-				type = gajim.get_transport_name_from_jid(jid,
+				jid = model[iter_][0].decode('utf-8')
+				type_ = gajim.get_transport_name_from_jid(jid,
 							use_config_setting = False)
-				if type:
-					identity = {'category': '_jid', 'type': type}
+				if type_:
+					identity = {'category': '_jid', 'type': type_}
 					klass = self.cache.get_browser([identity])
 					if klass:
 						self.browse_button.set_sensitive(True)
@@ -1399,13 +1398,13 @@ class ToplevelAgentBrowser(AgentBrowser):
 			fraction = float(self._progress) / float(self._total_items)
 			if self._progress >= self._total_items:
 				# We show the progressbar for just a bit before hiding it.
-				id = gobject.timeout_add_seconds(2, self._hide_progressbar_cb)
-				self._progressbar_sourceid = id
+				id_ = gobject.timeout_add_seconds(2, self._hide_progressbar_cb)
+				self._progressbar_sourceid = id_
 			else:
 				self.window.progressbar.show()
 				# Hide the progressbar if we're timing out anyways. (20 secs)
-				id = gobject.timeout_add_seconds(20, self._hide_progressbar_cb)
-				self._progressbar_sourceid = id
+				id_ = gobject.timeout_add_seconds(20, self._hide_progressbar_cb)
+				self._progressbar_sourceid = id_
 		self.window.progressbar.set_fraction(fraction)
 
 	def _hide_progressbar_cb(self, *args):
@@ -1437,33 +1436,33 @@ class ToplevelAgentBrowser(AgentBrowser):
 
 	def _find_category(self, cat, type_=None):
 		'''Looks up a category row and returns the iterator to it, or None.'''
-		cat, prio = self._friendly_category(cat, type_)
-		iter = self.model.get_iter_root()
-		while iter:
-			if self.model.get_value(iter, 3).decode('utf-8') == cat:
+		cat = self._friendly_category(cat, type_)[0]
+		iter_ = self.model.get_iter_root()
+		while iter_:
+			if self.model.get_value(iter_, 3).decode('utf-8') == cat:
 				break
-			iter = self.model.iter_next(iter)
-		if iter:
-			return iter
+			iter_ = self.model.iter_next(iter_)
+		if iter_:
+			return iter_
 		return None
 
 	def _find_item(self, jid, node):
-		iter = None
+		iter_ = None
 		cat_iter = self.model.get_iter_root()
-		while cat_iter and not iter:
-			iter = self.model.iter_children(cat_iter)
-			while iter:
-				cjid = self.model.get_value(iter, 0).decode('utf-8')
-				cnode = self.model.get_value(iter, 1).decode('utf-8')
+		while cat_iter and not iter_:
+			iter_ = self.model.iter_children(cat_iter)
+			while iter_:
+				cjid = self.model.get_value(iter_, 0).decode('utf-8')
+				cnode = self.model.get_value(iter_, 1).decode('utf-8')
 				if jid == cjid and node == cnode:
 					break
-				iter = self.model.iter_next(iter)
+				iter_ = self.model.iter_next(iter_)
 			cat_iter = self.model.iter_next(cat_iter)
-		if iter:
-			return iter
+		if iter_:
+			return iter_
 		return None
 
-	def _add_item(self, jid, node, item, force):
+	def _add_item(self, jid, node, parent_node, item, force):
 		# Row text
 		addr = get_agent_address(jid, node)
 		if 'name' in item:
@@ -1472,12 +1471,12 @@ class ToplevelAgentBrowser(AgentBrowser):
 			descr = "<b>%s</b>" % addr
 		# Guess which kind of service this is
 		identities = []
-		type = gajim.get_transport_name_from_jid(jid,
+		type_ = gajim.get_transport_name_from_jid(jid,
 					use_config_setting = False)
-		if type:
-			identity = {'category': '_jid', 'type': type}
+		if type_:
+			identity = {'category': '_jid', 'type': type_}
 			identities.append(identity)
-			cat_args = ('_jid', type)
+			cat_args = ('_jid', type_)
 		else:
 			# Put it in the 'other' category for now
 			cat_args = ('other',)
@@ -1487,10 +1486,10 @@ class ToplevelAgentBrowser(AgentBrowser):
 		cat = self._find_category(*cat_args)
 		if not cat:
 			cat = self._create_category(*cat_args)
-		self.model.append(cat, (item['jid'], item.get('node', ''), pix, descr, 1))
-		self._expand_all()
+		self.model.append(cat, (jid, node, pix, descr, 1))
+		gobject.idle_add(self._expand_all)
 		# Grab info on the service
-		self.cache.get_info(jid, node, self._agent_info, force = force)
+		self.cache.get_info(jid, node, self._agent_info, force=force)
 		self._update_progressbar()
 
 	def _update_item(self, iter_, jid, node, item):
@@ -1515,9 +1514,10 @@ class ToplevelAgentBrowser(AgentBrowser):
 
 		# Search for an icon and category we can display
 		pix = self.cache.get_icon(identities)
+		cat, type_ = None, None
 		for identity in identities:
 			try:
-				cat, type = identity['category'], identity['type']
+				cat, type_ = identity['category'], identity['type']
 			except KeyError:
 				continue
 			break
@@ -1540,14 +1540,13 @@ class ToplevelAgentBrowser(AgentBrowser):
 		if not self.model.iter_children(old_cat_iter):
 			self.model.remove(old_cat_iter)
 
-		cat_iter = self._find_category(cat, type)
+		cat_iter = self._find_category(cat, type_)
 		if not cat_iter:
-			cat_iter = self._create_category(cat, type)
+			cat_iter = self._create_category(cat, type_)
 		self.model.append(cat_iter, (jid, node, pix, descr, 0))
 		self._expand_all()
 
 	def _update_error(self, iter_, jid, node):
-		addr = get_agent_address(jid, node)
 		self.model[iter_][4] = 2
 		self._progress += 1
 		self._update_progressbar()
@@ -1557,6 +1556,7 @@ class MucBrowser(AgentBrowser):
 	def __init__(self, *args, **kwargs):
 		AgentBrowser.__init__(self, *args, **kwargs)
 		self.join_button = None
+		self.bookmark_button = None
 
 	def _create_treemodel(self):
 		# JID, node, name, users_int, users_str, description, fetched
@@ -1622,27 +1622,64 @@ class MucBrowser(AgentBrowser):
 		AgentBrowser._clean_treemodel(self)
 
 	def _add_actions(self):
+		self.bookmark_button = gtk.Button(label=_('_Bookmark'), use_underline=True)
+		self.bookmark_button.connect('clicked', self.on_bookmark_button_clicked)
+		self.window.action_buttonbox.add(self.bookmark_button)
+		self.bookmark_button.show_all()
 		self.join_button = gtk.Button(label=_('_Join'), use_underline=True)
 		self.join_button.connect('clicked', self.on_join_button_clicked)
 		self.window.action_buttonbox.add(self.join_button)
 		self.join_button.show_all()
 
 	def _clean_actions(self):
+		if self.bookmark_button:
+			self.bookmark_button.destroy()
+			self.bookmark_button = None
 		if self.join_button:
 			self.join_button.destroy()
 			self.join_button = None
 
-	def on_join_button_clicked(self, *args):
-		'''When we want to join a conference:
-		Ask specific informations about the selected agent and close the window'''
+	def on_bookmark_button_clicked(self, *args):
 		model, iter = self.window.services_treeview.get_selection().get_selected()
 		if not iter:
 			return
-		service = model[iter][0].decode('utf-8')
-		room = model[iter][1].decode('utf-8')
+		name = gajim.config.get_per('accounts', self.account, 'name')
+		room_jid = model[iter][0].decode('utf-8')
+		bm = {
+			'name': room_jid.split('@')[0],
+			'jid': room_jid,
+			'autojoin': '0',
+			'minimize': '0',
+			'password': '',
+			'nick': name
+		}
+
+		for bookmark in gajim.connections[self.account].bookmarks:
+			if bookmark['jid'] == bm['jid']:
+				dialogs.ErrorDialog(
+					_('Bookmark already set'),
+					_('Group Chat "%s" is already in your bookmarks.') % bm['jid'])
+				return
+
+		gajim.connections[self.account].bookmarks.append(bm)
+		gajim.connections[self.account].store_bookmarks()
+
+		gajim.interface.roster.set_actions_menu_needs_rebuild()
+
+		dialogs.InformationDialog(
+				_('Bookmark has been added successfully'),
+				_('You can manage your bookmarks via Actions menu in your roster.'))
+
+	def on_join_button_clicked(self, *args):
+		'''When we want to join a conference:
+		Ask specific informations about the selected agent and close the window'''
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if not iter_:
+			return
+		service = model[iter_][0].decode('utf-8')
+		room = model[iter_][1].decode('utf-8')
 		if 'join_gc' not in gajim.interface.instances[self.account]:
 			try:
-				room_jid = '%s@%s' % (service, room)
 				dialogs.JoinGroupchatWindow(self.account, service)
 			except GajimGeneralException:
 				pass
@@ -1651,8 +1688,10 @@ class MucBrowser(AgentBrowser):
 		self.window.destroy(chain = True)
 
 	def update_actions(self):
+		sens = self.window.services_treeview.get_selection().count_selected_rows()
+		if self.bookmark_button:
+			self.bookmark_button.set_sensitive(sens > 0)
 		if self.join_button:
-			sens = self.window.services_treeview.get_selection().count_selected_rows()
 			self.join_button.set_sensitive(sens > 0)
 
 	def default_action(self):
@@ -1682,12 +1721,12 @@ class MucBrowser(AgentBrowser):
 		# We have to do this in a pygtk <2.8 compatible way :/
 		#start, end = self.window.services_treeview.get_visible_range()
 		rect = view.get_visible_rect()
-		iter = end = None
+		iter_ = end = None
 		# Top row
 		try:
 			sx, sy = view.tree_to_widget_coords(rect.x, rect.y)
 			spath = view.get_path_at_pos(sx, sy)[0]
-			iter = self.model.get_iter(spath)
+			iter_ = self.model.get_iter(spath)
 		except TypeError:
 			self._fetch_source = None
 			return
@@ -1702,14 +1741,14 @@ class MucBrowser(AgentBrowser):
 		except TypeError:
 			# We're at the end of the model, we can leave end=None though.
 			pass
-		while iter and self.model.get_path(iter) != end:
-			if not self.model.get_value(iter, 6):
-				jid = self.model.get_value(iter, 0).decode('utf-8')
-				node = self.model.get_value(iter, 1).decode('utf-8')
+		while iter_ and self.model.get_path(iter_) != end:
+			if not self.model.get_value(iter_, 6):
+				jid = self.model.get_value(iter_, 0).decode('utf-8')
+				node = self.model.get_value(iter_, 1).decode('utf-8')
 				self.cache.get_info(jid, node, self._agent_info)
 				self._fetch_source = True
 				return
-			iter = self.model.iter_next(iter)
+			iter_ = self.model.iter_next(iter_)
 		self._fetch_source = None
 
 	def _channel_altinfo(self, jid, node, items, name = None):
@@ -1730,17 +1769,17 @@ class MucBrowser(AgentBrowser):
 				self._fetch_source = None
 				return
 		else:
-			iter = self._find_item(jid, node)
-			if iter:
+			iter_ = self._find_item(jid, node)
+			if iter_:
 				if name:
-					self.model[iter][2] = name
-				self.model[iter][3] = len(items) # The number of users
-				self.model[iter][4] = str(len(items)) # The number of users
-				self.model[iter][6] = True
+					self.model[iter_][2] = name
+				self.model[iter_][3] = len(items) # The number of users
+				self.model[iter_][4] = str(len(items)) # The number of users
+				self.model[iter_][6] = True
 		self._fetch_source = None
 		self._query_visible()
 
-	def _add_item(self, jid, node, item, force):
+	def _add_item(self, jid, node, parent_node, item, force):
 		self.model.append((jid, node, item.get('name', ''), -1, '', '', False))
 		if not self._fetch_source:
 			self._fetch_source = gobject.idle_add(self._start_info_query)
@@ -1838,10 +1877,10 @@ class DiscussionGroupsBrowser(AgentBrowser):
 
 	def _add_items(self, jid, node, items, force):
 		for item in items:
-			jid = item['jid']
-			node = item.get('node', '')
+			jid_ = item['jid']
+			node_ = item.get('node', '')
 			self._total_items += 1
-			self._add_item(jid, node, item, force)
+			self._add_item(jid_, node_, node, item, force)
 
 	def _in_list_foreach(self, model, path, iter_, node):
 		if model[path][1] == node:
@@ -1852,7 +1891,7 @@ class DiscussionGroupsBrowser(AgentBrowser):
 		self.model.foreach(self._in_list_foreach, node)
 		return self.in_list
 
-	def _add_item(self, jid, node, item, force):
+	def _add_item(self, jid, node, parent_node, item, force):
 		''' Called when we got basic information about new node from query.
 		Show the item. '''
 		name = item.get('name', '')
@@ -1867,12 +1906,7 @@ class DiscussionGroupsBrowser(AgentBrowser):
 		name = gobject.markup_escape_text(name)
 		name = '<b>%s</b>' % name
 
-		node_splitted = node.split('/')
-		parent_iter = None
-		while len(node_splitted) > 1:
-			parent_node = node_splitted.pop(0)
-			parent_iter = self._get_child_iter(parent_iter, parent_node)
-			node_splitted[0] = parent_node + '/' + node_splitted[0]
+		parent_iter = self._get_iter(parent_node)
 		if not self._in_list(node):
 			self.model.append(parent_iter, (jid, node, name, dunno, subscribed))
 			self.cache.get_items(jid, node, self._add_items, force = force,
@@ -1886,6 +1920,16 @@ class DiscussionGroupsBrowser(AgentBrowser):
 			child_iter = self.model.iter_next(child_iter)
 		return None
 
+	def _get_iter(self, node):
+		''' Look for an iter with the given node '''
+		self.found_iter = None
+		def is_node(model, path, iter, node):
+			if model[iter][1] == node:
+				self.found_iter = iter
+				return True
+		self.model.foreach(is_node, node)
+		return self.found_iter
+
 	def _add_actions(self):
 		self.post_button = gtk.Button(label=_('New post'), use_underline=True)
 		self.post_button.set_sensitive(False)
@@ -1924,43 +1968,43 @@ class DiscussionGroupsBrowser(AgentBrowser):
 		# we have nothing to do if we don't have buttons...
 		if self.subscribe_button is None: return
 
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		if not iter or self.subscriptions is None:
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if not iter_ or self.subscriptions is None:
 			# no item selected or no subscriptions info, all buttons are insensitive
 			self.post_button.set_sensitive(False)
 			self.subscribe_button.set_sensitive(False)
 			self.unsubscribe_button.set_sensitive(False)
 		else:
-			subscribed = model.get_value(iter, 4) # 4 = subscribed?
+			subscribed = model.get_value(iter_, 4) # 4 = subscribed?
 			self.post_button.set_sensitive(subscribed)
 			self.subscribe_button.set_sensitive(not subscribed)
 			self.unsubscribe_button.set_sensitive(subscribed)
 
 	def on_post_button_clicked(self, widget):
 		'''Called when 'post' button is pressed. Open window to create post'''
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		if iter is None: return
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if iter_ is None: return
 
-		groupnode = model.get_value(iter, 1)	# 1 = groupnode
+		groupnode = model.get_value(iter_, 1)	# 1 = groupnode
 
 		groups.GroupsPostWindow(self.account, self.jid, groupnode)
 
 	def on_subscribe_button_clicked(self, widget):
 		'''Called when 'subscribe' button is pressed. Send subscribtion request.'''
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		if iter is None: return
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if iter_ is None: return
 
-		groupnode = model.get_value(iter, 1)	# 1 = groupnode
+		groupnode = model.get_value(iter_, 1)	# 1 = groupnode
 
 		gajim.connections[self.account].send_pb_subscribe(self.jid, groupnode, self._subscribeCB, groupnode)
-		
+
 	def on_unsubscribe_button_clicked(self, widget):
 		'''Called when 'unsubscribe' button is pressed. Send unsubscription request.'''
-		model, iter = self.window.services_treeview.get_selection().get_selected()
-		if iter is None: return
+		model, iter_ = self.window.services_treeview.get_selection().get_selected()
+		if iter_ is None: return
+
+		groupnode = model.get_value(iter_, 1) # 1 = groupnode
 
-		groupnode = model.get_value(iter, 1) # 1 = groupnode
-		
 		gajim.connections[self.account].send_pb_unsubscribe(self.jid, groupnode, self._unsubscribeCB, groupnode)
 
 	def _subscriptionsCB(self, conn, request):
@@ -1969,7 +2013,7 @@ class DiscussionGroupsBrowser(AgentBrowser):
 		try:
 			subscriptions = request.getTag('pubsub').getTag('subscriptions')
 		except Exception:
-			return 
+			return
 
 		groups = set()
 		for child in subscriptions.getTags('subscription'):
