@@ -145,21 +145,20 @@ class HistoryWindow:
 
 		# Add all jids in logs.db:
 		db_jids = gajim.logger.get_jids_in_db()
-		self.completion_dict = dict.fromkeys(db_jids)
+		completion_dict = dict.fromkeys(db_jids)
 
 		self.accounts_seen_online = gajim.contacts.get_accounts()[:]
 
 		# Enhance contacts of online accounts with contact. Needed for mapping below
 		for account in self.accounts_seen_online:
-			self.completion_dict.update(
-				helpers.get_contact_dict_for_account(account))
+			completion_dict.update(helpers.get_contact_dict_for_account(account))
 
 		muc_active_img = gtkgui_helpers.load_icon('muc_active')
 		contact_img = gajim.interface.jabber_state_images['16']['online']
 		muc_active_pix = muc_active_img.get_pixbuf()
 		contact_pix = contact_img.get_pixbuf()
 
-		keys = self.completion_dict.keys()
+		keys = completion_dict.keys()
 		# Move the actual jid at first so we load history faster
 		actual_jid = self.jid_entry.get_text().decode('utf-8')
 		if actual_jid in keys:
@@ -171,7 +170,7 @@ class HistoryWindow:
 		# Warning : This for is time critical with big DB
 		for key in keys:
 			completed = key
-			contact = self.completion_dict[completed]
+			contact = completion_dict[completed]
 			if contact:
 				info_name = contact.get_shown_name()
 				info_completion = info_name
@@ -244,7 +243,7 @@ class HistoryWindow:
 		self.results_window.set_property('visible', False)
 
 	def on_jid_entry_focus(self, widget, event):
-			widget.select_region(0, -1) # select text
+		widget.select_region(0, -1) # select text
 
 	def _load_history(self, jid_or_name, account = None):
 		'''Load history for the given jid/name and show it'''
