@@ -488,7 +488,9 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
 			# Wait maximum 1s for connection
 			socket.setdefaulttimeout(1)
 			try:
-				f = urllib2.urlopen(attrs['src'])
+				req = urllib2.Request(attrs['src'])
+				req.add_header('User-Agent', 'Gajim ' + gajim.version)
+				f = urllib2.urlopen(req)
 			except Exception, ex:
 				gajim.log.debug('Error loading image %s ' % attrs['src']  + str(ex))
 				pixbuf = None
@@ -804,6 +806,10 @@ class HtmlTextView(gtk.TextView):
 		self.connect('motion-notify-event', self.__motion_notify_event)
 		self.connect('leave-notify-event', self.__leave_event)
 		self.connect('enter-notify-event', self.__motion_notify_event)
+		self.connect('realize', self.on_html_text_view_realized)
+		self.connect('unrealize', self.on_html_text_view_unrealized)
+		self.connect('copy-clipboard', self.on_html_text_view_copy_clipboard)
+		self.get_buffer().connect_after('mark-set', self.on_text_buffer_mark_set)
 		self.get_buffer().create_tag('eol', scale = pango.SCALE_XX_SMALL)
 		self.tooltip = tooltips.BaseTooltip()
 		self.config = gajim.config
@@ -873,7 +879,45 @@ class HtmlTextView(gtk.TextView):
 		#if not eob.starts_line():
 		#    buffer_.insert(eob, '\n')
 
+	def on_html_text_view_copy_clipboard(self, unused_data):
+		clipboard = self.get_clipboard(gtk.gdk.SELECTION_CLIPBOARD)
+		clipboard.set_text(self.get_selected_text())
+		self.emit_stop_by_name('copy-clipboard')
 
+	def on_html_text_view_realized(self, unused_data):
+		self.get_buffer().remove_selection_clipboard(self.get_clipboard(gtk.gdk.SELECTION_PRIMARY))
+
+	def on_html_text_view_unrealized(self, unused_data):
+		self.get_buffer().add_selection_clipboard(self.get_clipboard(gtk.gdk.SELECTION_PRIMARY))
+
+	def on_text_buffer_mark_set(self, location, mark, unused_data):
+		bounds = self.get_buffer().get_selection_bounds()
+		if bounds:
+			# textview can be hidden while we add a new line in it.
+			if self.has_screen():
+				clipboard = self.get_clipboard(gtk.gdk.SELECTION_PRIMARY)
+				clipboard.set_text(self.get_selected_text())
+
+	def get_selected_text(self):
+		bounds = self.get_buffer().get_selection_bounds()
+		selection = ''
+		if bounds:
+			(search_iter, end) = bounds
+
+			while (search_iter.compare(end)):
+				character = search_iter.get_char()
+				if character == u'\ufffc':
+					anchor = search_iter.get_child_anchor()
+					if anchor:
+						text = anchor.get_data('plaintext')
+						if text:
+							selection+=text
+					else:
+						selection+=character
+				else:
+					selection+=character
+				search_iter.forward_char()
+		return selection
 
 change_cursor = None
 
