@@ -24,6 +24,7 @@
 import sys
 from common import gajim
 
+
 def device_now_active(self, *args):
 	'''For Network Manager 0.6'''
 	for connection in gajim.connections.itervalues():
@@ -38,41 +39,44 @@ def device_no_longer_active(self, *args):
 		'listen_to_network_manager') and connection.connected > 1:
 			connection._disconnectedReconnCB()
 
-def state_changed(state): 
+def state_changed(state):
 	'''For Network Manager 0.7'''
-	if props.Get("org.freedesktop.NetworkManager", "State") == 3: 
-		for connection in gajim.connections.itervalues(): 
-			if gajim.config.get_per('accounts', connection.name, 
-			'listen_to_network_manager') and connection.time_to_reconnect: 
-				connection._reconnect() 
-	else: 
-		for connection in gajim.connections.itervalues(): 
-			if gajim.config.get_per('accounts', connection.name, 
-			'listen_to_network_manager') and connection.connected > 1: 
+	if props.Get("org.freedesktop.NetworkManager", "State") == 3:
+		for connection in gajim.connections.itervalues():
+			if gajim.config.get_per('accounts', connection.name,
+			'listen_to_network_manager') and connection.time_to_reconnect:
+				connection._reconnect()
+	else:
+		for connection in gajim.connections.itervalues():
+			if gajim.config.get_per('accounts', connection.name,
+			'listen_to_network_manager') and connection.connected > 1:
 				connection._disconnectedReconnCB()
 
 supported = False
 
-if sys.platform == 'darwin':
-	supported = True
-else:
+from common import dbus_support
+
+if dbus_support.supported:
+	import dbus
+	import dbus.glib
+
 	try:
 		from common.dbus_support import system_bus
 
 		bus = system_bus.bus()
 
 		if 'org.freedesktop.NetworkManager' in bus.list_names():
-			nm_object = bus.get_object('org.freedesktop.NetworkManager', 
-				'/org/freedesktop/NetworkManager') 
-			props = dbus.Interface(nm_object,"org.freedesktop.DBus.Properties") 
-			bus.add_signal_receiver(state_changed, 
-				'StateChanged', 
-				'org.freedesktop.NetworkManager', 
-				'org.freedesktop.NetworkManager', 
+			nm_object = bus.get_object('org.freedesktop.NetworkManager',
+				'/org/freedesktop/NetworkManager')
+			props = dbus.Interface(nm_object,"org.freedesktop.DBus.Properties")
+			bus.add_signal_receiver(state_changed,
+				'StateChanged',
+				'org.freedesktop.NetworkManager',
+				'org.freedesktop.NetworkManager',
 				'/org/freedesktop/NetworkManager')
 			supported = True
 
-	except Exception:
+	except dbus.DBusException:
 		try:
 			if 'org.freedesktop.NetworkManager' in bus.list_names():
 				supported = True
