@@ -27,6 +27,7 @@
 ##
 
 import gobject
+import gtk
 import os
 
 from common import gajim
@@ -90,7 +91,7 @@ def get_dbus_struct(obj):
 			return DBUS_NONE()
 		return result
 	# unknown type
-	return DBUS_NONE() 
+	return DBUS_NONE()
 
 class Remote:
 	def __init__(self):
@@ -171,6 +172,10 @@ class SignalObject(dbus.service.Object):
 	def OsInfo(self, account_and_array):
 		pass
 
+	@dbus.service.signal(INTERFACE, signature='av')
+	def EntityTime(self, account_and_array):
+		pass
+
 	@dbus.service.signal(INTERFACE, signature='av')
 	def GCPresence(self, account_and_array):
 		pass
@@ -268,7 +273,7 @@ class SignalObject(dbus.service.Object):
 
 	@dbus.service.method(INTERFACE, in_signature='sss', out_signature='b')
 	def send_file(self, file_path, jid, account):
-		'''send file, located at 'file_path' to 'jid', using account 
+		'''send file, located at 'file_path' to 'jid', using account
 		(optional) 'account' '''
 		jid = self._get_real_jid(jid, account)
 		connected_account, contact = self._get_account_and_contact(account, jid)
@@ -325,13 +330,12 @@ class SignalObject(dbus.service.Object):
 			return DBUS_BOOLEAN(True)
 		return DBUS_BOOLEAN(False)
 
-	@dbus.service.method(INTERFACE, in_signature='ss', out_signature='b')
-	def open_chat(self, jid, account):
+	@dbus.service.method(INTERFACE, in_signature='sss', out_signature='b')
+	def open_chat(self, jid, account, message):
 		'''Shows the tabbed window for new message to 'jid', using account
 		(optional) 'account' '''
 		if not jid:
-			raise MissingArgument
-			return DBUS_BOOLEAN(False)
+			raise dbus_support.MissingArgument()
 		jid = self._get_real_jid(jid, account)
 		try:
 			jid = helpers.parse_jid(jid)
@@ -359,7 +363,7 @@ class SignalObject(dbus.service.Object):
 					break
 				# we send the message to jid not in roster, because account is
 				# specified, or there is only one account
-				elif account: 
+				elif account:
 					connected_account = acct
 				elif first_connected_acct is None:
 					first_connected_acct = acct
@@ -369,12 +373,12 @@ class SignalObject(dbus.service.Object):
 			connected_account = first_connected_acct
 
 		if connected_account:
-			gajim.interface.new_chat_from_jid(connected_account, jid)
+			gajim.interface.new_chat_from_jid(connected_account, jid, message)
 			# preserve the 'steal focus preservation'
 			win = gajim.interface.msg_win_mgr.get_window(jid,
 				connected_account).window
 			if win.get_property('visible'):
-				win.window.focus()
+				win.window.focus(gtk.get_current_event_time())
 			return DBUS_BOOLEAN(True)
 		return DBUS_BOOLEAN(False)
 
@@ -382,7 +386,7 @@ class SignalObject(dbus.service.Object):
 	def change_status(self, status, message, account):
 		''' change_status(status, message, account). account is optional -
 		if not specified status is changed for all accounts. '''
-		if status not in ('offline', 'online', 'chat', 
+		if status not in ('offline', 'online', 'chat',
 			'away', 'xa', 'dnd', 'invisible'):
 			return DBUS_BOOLEAN(False)
 		if account:
@@ -398,6 +402,31 @@ class SignalObject(dbus.service.Object):
 					status, message)
 		return DBUS_BOOLEAN(False)
 
+	@dbus.service.method(INTERFACE, in_signature='ss', out_signature='')
+	def set_priority(self, prio, account):
+		''' set_priority(prio, account). account is optional -
+		if not specified priority is changed for all accounts. that are synced
+		with global status'''
+		if account:
+			gajim.config.set_per('accounts', account, 'priority', prio)
+			show = gajim.SHOW_LIST[gajim.connections[account].connected]
+			status = gajim.connections[account].status
+			gobject.idle_add(gajim.connections[account].change_status, show,
+				status)
+		else:
+			# account not specified, so change prio of all accounts
+			for acc in gajim.contacts.get_accounts():
+				if not gajim.account_is_connected(acc):
+					continue
+				if not gajim.config.get_per('accounts', acc,
+				'sync_with_global_status'):
+					continue
+				gajim.config.set_per('accounts', acc, 'priority', prio)
+				show = gajim.SHOW_LIST[gajim.connections[acc].connected]
+				status = gajim.connections[acc].status
+				gobject.idle_add(gajim.connections[acc].change_status, show,
+					status)
+
 	@dbus.service.method(INTERFACE, in_signature='', out_signature='')
 	def show_next_pending_event(self):
 		'''Show the window(s) with next pending event in tabbed/group chats.'''
@@ -411,8 +440,7 @@ class SignalObject(dbus.service.Object):
 		if not isinstance(jid, unicode):
 			jid = unicode(jid)
 		if not jid:
-			raise MissingArgument
-			return DBUS_DICT_SV()
+			raise dbus_support.MissingArgument()
 		jid = self._get_real_jid(jid)
 
 		cached_vcard = gajim.connections.values()[0].get_cached_vcard(jid)
@@ -480,7 +508,7 @@ class SignalObject(dbus.service.Object):
 			win.present()
 			# preserve the 'steal focus preservation'
 			if self._is_first():
-				win.window.focus()
+				win.window.focus(gtk.get_current_event_time())
 			else:
 				win.window.focus(long(time()))
 
@@ -551,7 +579,7 @@ class SignalObject(dbus.service.Object):
 		if account:
 			if account in gajim.connections and \
 				gajim.connections[account].connected > 1:
-				# if given account is active, use it 
+				# if given account is active, use it
 				AddNewContactWindow(account = account, jid = jid)
 			else:
 				# wrong account
