@@ -90,12 +90,16 @@ class VcardWindow:
 			gc_contact.room_jid, account)
 			if gc_contact.jid and not gc_control.is_anonymous:
 				self.real_jid = gc_contact.jid
+				self.real_jid_for_vcard = gc_contact.jid
 				if gc_contact.resource:
 					self.real_jid += '/' + gc_contact.resource
 			else:
 				self.real_jid = gc_contact.get_full_jid()
+				self.real_jid_for_vcard = self.real_jid
+			self.real_resource = gc_contact.name
 		else:
 			self.real_jid = contact.get_full_jid()
+			self.real_resource = contact.resource
 
 		puny_jid = helpers.sanitize_filename(contact.jid)
 		local_avatar_basepath = os.path.join(gajim.AVATAR_PATH, puny_jid) + \
@@ -112,14 +116,15 @@ class VcardWindow:
 		self.avatar_encoded = None
 		self.vcard_arrived = False
 		self.os_info_arrived = False
+		self.entity_time_arrived = False
 		self.update_progressbar_timeout_id = gobject.timeout_add(100,
 			self.update_progressbar)
 
 		self.fill_jabber_page()
 		annotations = gajim.connections[self.account].annotations
 		if self.contact.jid in annotations:
-			buffer = self.xml.get_widget('textview_annotation').get_buffer()
-			buffer.set_text(annotations[self.contact.jid])
+			buffer_ = self.xml.get_widget('textview_annotation').get_buffer()
+			buffer_.set_text(annotations[self.contact.jid])
 
 		self.xml.signal_autoconnect(self)
 		self.window.show_all()
@@ -133,9 +138,9 @@ class VcardWindow:
 		if self.update_progressbar_timeout_id is not None:
 			gobject.source_remove(self.update_progressbar_timeout_id)
 		del gajim.interface.instances[self.account]['infos'][self.contact.jid]
-		buffer = self.xml.get_widget('textview_annotation').get_buffer()
-		annotation = buffer.get_text(buffer.get_start_iter(),
-			buffer.get_end_iter())
+		buffer_ = self.xml.get_widget('textview_annotation').get_buffer()
+		annotation = buffer_.get_text(buffer_.get_start_iter(),
+			buffer_.get_end_iter())
 		connection = gajim.connections[self.account]
 		if annotation != connection.annotations.get(self.contact.jid, ''):
 			connection.annotations[self.contact.jid] = annotation
@@ -155,7 +160,7 @@ class VcardWindow:
 				self.contact.jid, self.account, self.contact.get_shown_name() +
 				'.jpeg')
 			menu.append(menuitem)
-			menu.connect('selection-done', lambda w:w.destroy())	
+			menu.connect('selection-done', lambda w:w.destroy())
 			# show the menu
 			menu.show_all()
 			menu.popup(None, None, None, event.button, event.time)
@@ -163,13 +168,8 @@ class VcardWindow:
 	def set_value(self, entry_name, value):
 		try:
 			if value and entry_name == 'URL_label':
-				if gtk.pygtk_version >= (2, 10, 0) and gtk.gtk_version >= (2, 10, 0):
-					widget = gtk.LinkButton(value, value)
-					widget.set_alignment(0, 0)
-				else:
-					widget = gtk.Label(value)
-					widget.set_alignment(0, 0)
-					widget.set_selectable(True)
+				widget = gtk.LinkButton(value, value)
+				widget.set_alignment(0, 0)
 				widget.show()
 				table = self.xml.get_widget('personal_info_table')
 				table.attach(widget, 1, 4, 3, 4, yoptions = 0)
@@ -215,7 +215,7 @@ class VcardWindow:
 
 	def test_remove_progressbar(self):
 		if self.update_progressbar_timeout_id is not None and \
-		self.vcard_arrived and self.os_info_arrived:
+		self.vcard_arrived and self.os_info_arrived and self.entity_time_arrived:
 			gobject.source_remove(self.update_progressbar_timeout_id)
 			self.progressbar.hide()
 			self.update_progressbar_timeout_id = None
@@ -250,6 +250,26 @@ class VcardWindow:
 		self.os_info_arrived = True
 		self.test_remove_progressbar()
 
+	def set_entity_time(self, resource, time_info):
+		if self.xml.get_widget('information_notebook').get_n_pages() < 5:
+			return
+		i = 0
+		time_s = ''
+		while i in self.time_info:
+			if not self.time_info[i]['resource'] or \
+			self.time_info[i]['resource'] == resource:
+				self.time_info[i]['time'] = time_info
+			if i > 0:
+				time_s += '\n'
+			time_s += self.time_info[i]['time']
+			i += 1
+
+		if time_s == '':
+			time_s = Q_('?Time:Unknown')
+		self.xml.get_widget('time_label').set_text(time_s)
+		self.entity_time_arrived = True
+		self.test_remove_progressbar()
+
 	def fill_status_label(self):
 		if self.xml.get_widget('information_notebook').get_n_pages() < 5:
 			return
@@ -289,18 +309,16 @@ class VcardWindow:
 		status_label.set_max_width_chars(15)
 		status_label.set_text(stats)
 
-		tip = gtk.Tooltips()
 		status_label_eventbox = self.xml.get_widget('status_label_eventbox')
-		tip.set_tip(status_label_eventbox, stats)
+		status_label_eventbox.set_tooltip_text(stats)
 
 	def fill_jabber_page(self):
-		tooltips = gtk.Tooltips()
 		self.xml.get_widget('nickname_label').set_markup(
 			'<b><span size="x-large">' +
 			self.contact.get_shown_name() +
 			'</span></b>')
 		self.xml.get_widget('jid_label').set_text(self.contact.jid)
-		
+
 		subscription_label = self.xml.get_widget('subscription_label')
 		ask_label = self.xml.get_widget('ask_label')
 		if self.gc_contact:
@@ -323,7 +341,7 @@ class VcardWindow:
 				tt_text = _("You and the contact are interested in each other's presence information")
 			else: # None
 				tt_text = _("You are not interested in the contact's presence, and neither he/she is interested in yours")
-			tooltips.set_tip(eb, tt_text)
+			eb.set_tooltip_text(tt_text)
 
 			uf_ask = helpers.get_uf_ask(self.contact.ask)
 			ask_label.set_text(uf_ask)
@@ -332,7 +350,7 @@ class VcardWindow:
 				tt_text = _("You are waiting contact's answer about your subscription request")
 			else:
 				tt_text = _("There is no pending subscription request.")
-			tooltips.set_tip(eb, tt_text)
+			eb.set_tooltip_text(tt_text)
 
 		resources = '%s (%s)' % (self.contact.resource, unicode(
 			self.contact.priority))
@@ -364,8 +382,24 @@ class VcardWindow:
 			else:
 				gobject.idle_add(gajim.connections[self.account].request_os_info,
 					self.contact.jid, self.contact.resource)
-		self.os_info = {0: {'resource': self.contact.resource, 'client': '',
+
+		# do not wait for entity_time if contact is not connected or has error
+		# additional check for observer is needed, as show is offline for him
+		if self.contact.show in ('offline', 'error')\
+		and not self.contact.is_observer():
+			self.entity_time_arrived = True
+		else: # Request entity time if contact is connected
+			if self.gc_contact:
+				j, r = gajim.get_room_and_nick_from_fjid(self.real_jid)
+				gobject.idle_add(gajim.connections[self.account].\
+					request_entity_time, j, r, self.contact.jid)
+			else:
+				gobject.idle_add(gajim.connections[self.account].\
+					request_entity_time, self.contact.jid, self.contact.resource)
+
+		self.os_info = {0: {'resource': self.real_resource, 'client': '',
 			'os': ''}}
+		self.time_info = {0: {'resource': self.real_resource, 'time': ''}}
 		i = 1
 		contact_list = gajim.contacts.get_contacts(self.account, self.contact.jid)
 		if contact_list:
@@ -379,23 +413,23 @@ class VcardWindow:
 						gobject.idle_add(
 							gajim.connections[self.account].request_os_info, c.jid,
 							c.resource)
+						gobject.idle_add(gajim.connections[self.account].\
+							request_entity_time, c.jid, c.resource)
 					self.os_info[i] = {'resource': c.resource, 'client': '',
 						'os': ''}
+					self.time_info[i] = {'resource': c.resource, 'time': ''}
 					i += 1
+
 		self.xml.get_widget('resource_prio_label').set_text(resources)
 		resource_prio_label_eventbox = self.xml.get_widget(
 			'resource_prio_label_eventbox')
-		tooltips.set_tip(resource_prio_label_eventbox, uf_resources)
+		resource_prio_label_eventbox.set_tooltip_text(uf_resources)
 
 		self.fill_status_label()
 
 		if self.gc_contact:
 			# If we know the real jid, remove the resource from vcard request
-			if self.gc_contact.jid:
-				jid = self.gc_contact.jid
-			else:
-				jid = self.real_jid
-			gajim.connections[self.account].request_vcard(jid,
+			gajim.connections[self.account].request_vcard(self.real_jid_for_vcard,
 				self.gc_contact.get_full_jid())
 		else:
 			gajim.connections[self.account].request_vcard(self.contact.jid)
@@ -440,7 +474,7 @@ class ZeroconfVcardWindow:
 				self.contact.jid, self.account, self.contact.get_shown_name() +
 				'.jpeg')
 			menu.append(menuitem)
-			menu.connect('selection-done', lambda w:w.destroy())	
+			menu.connect('selection-done', lambda w:w.destroy())
 			# show the menu
 			menu.show_all()
 			menu.popup(None, None, None, event.button, event.time)
@@ -448,13 +482,8 @@ class ZeroconfVcardWindow:
 	def set_value(self, entry_name, value):
 		try:
 			if value and entry_name == 'URL_label':
-				if gtk.pygtk_version >= (2, 10, 0) and gtk.gtk_version >= (2, 10, 0):
-					widget = gtk.LinkButton(value, value)
-					widget.set_alignment(0, 0)
-				else:
-					widget = gtk.Label(value)
-					widget.set_selectable(True)
-					widget.set_alignment(0, 0)
+				widget = gtk.LinkButton(value, value)
+				widget.set_alignment(0, 0)
 				table = self.xml.get_widget('personal_info_table')
 				table.attach(widget, 1, 4, 3, 4, yoptions = 0)
 			else:
@@ -488,12 +517,10 @@ class ZeroconfVcardWindow:
 		status_label.set_max_width_chars(15)
 		status_label.set_text(stats)
 
-		tip = gtk.Tooltips()
 		status_label_eventbox = self.xml.get_widget('status_label_eventbox')
-		tip.set_tip(status_label_eventbox, stats)
-	
+		status_label_eventbox.set_tooltip_text(stats)
+
 	def fill_contact_page(self):
-		tooltips = gtk.Tooltips()
 		self.xml.get_widget('nickname_label').set_markup(
 			'<b><span size="x-large">' +
 			self.contact.get_shown_name() +
@@ -514,12 +541,12 @@ class ZeroconfVcardWindow:
 		self.xml.get_widget('resource_prio_label').set_text(resources)
 		resource_prio_label_eventbox = self.xml.get_widget(
 			'resource_prio_label_eventbox')
-		tooltips.set_tip(resource_prio_label_eventbox, uf_resources)
+		resource_prio_label_eventbox.set_tooltip_text(uf_resources)
 
 		self.fill_status_label()
 
 	#	gajim.connections[self.account].request_vcard(self.contact.jid, self.is_fake)
-	
+
 	def fill_personal_page(self):
 		contact = gajim.connections[gajim.ZEROCONF_ACC_NAME].roster.getItem(self.contact.jid)
 		for key in ('1st', 'last', 'jid', 'email'):
